#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.remote."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import sys
import types
import unittest
from wsgiref import headers

from protorpc import descriptor
from protorpc import message_types
from protorpc import messages
from protorpc import protobuf
from protorpc import protojson
from protorpc import remote
from protorpc import test_util
from protorpc import transport

import mox


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = remote


class Request(messages.Message):
  """Test request message."""

  value = messages.StringField(1)


class Response(messages.Message):
  """Test response message."""

  value = messages.StringField(1)


class MyService(remote.Service):

  @remote.method(Request, Response)
  def remote_method(self, request):
    response = Response()
    response.value = request.value
    return response


class SimpleRequest(messages.Message):
  """Simple request message type used for tests."""

  param1 = messages.StringField(1)
  param2 = messages.StringField(2)


class SimpleResponse(messages.Message):
  """Simple response message type used for tests."""


class BasicService(remote.Service):
  """A basic service with decorated remote method."""

  def __init__(self):
    self.request_ids = []

  @remote.method(SimpleRequest, SimpleResponse)
  def remote_method(self, request):
    """BasicService remote_method docstring."""
    self.request_ids.append(id(request))
    return SimpleResponse()


class RpcErrorTest(test_util.TestCase):

  def testFromStatus(self):
    for state in remote.RpcState:
      exception = remote.RpcError.from_state
    self.assertEquals(remote.ServerError,
                      remote.RpcError.from_state('SERVER_ERROR'))


class ApplicationErrorTest(test_util.TestCase):

  def testErrorCode(self):
    self.assertEquals('blam',
                      remote.ApplicationError('an error', 'blam').error_name)

  def testStr(self):
    self.assertEquals('an error', str(remote.ApplicationError('an error', 1)))

  def testRepr(self):
    self.assertEquals("ApplicationError('an error', 1)",
                      repr(remote.ApplicationError('an error', 1)))

    self.assertEquals("ApplicationError('an error')",
                      repr(remote.ApplicationError('an error')))


class MethodTest(test_util.TestCase):
  """Test remote method decorator."""

  def testMethod(self):
    """Test use of remote decorator."""
    self.assertEquals(SimpleRequest,
                      BasicService.remote_method.remote.request_type)
    self.assertEquals(SimpleResponse,
                      BasicService.remote_method.remote.response_type)
    self.assertTrue(isinstance(BasicService.remote_method.remote.method,
                               types.FunctionType))

  def testMethodMessageResolution(self):
    """Test use of remote decorator to resolve message types by name."""
    class OtherService(remote.Service):

      @remote.method('SimpleRequest', 'SimpleResponse')
      def remote_method(self, request):
        pass

    self.assertEquals(SimpleRequest,
                      OtherService.remote_method.remote.request_type)
    self.assertEquals(SimpleResponse,
                      OtherService.remote_method.remote.response_type)

  def testMethodMessageResolution_NotFound(self):
    """Test failure to find message types."""
    class OtherService(remote.Service):

      @remote.method('NoSuchRequest', 'NoSuchResponse')
      def remote_method(self, request):
        pass

    self.assertRaisesWithRegexpMatch(
      messages.DefinitionNotFoundError,
      'Could not find definition for NoSuchRequest',
      getattr,
      OtherService.remote_method.remote,
      'request_type')

    self.assertRaisesWithRegexpMatch(
      messages.DefinitionNotFoundError,
      'Could not find definition for NoSuchResponse',
      getattr,
      OtherService.remote_method.remote,
      'response_type')

  def testInvocation(self):
    """Test that invocation passes request through properly."""
    service = BasicService()
    request = SimpleRequest()
    self.assertEquals(SimpleResponse(), service.remote_method(request))
    self.assertEquals([id(request)], service.request_ids)

  def testInvocation_WrongRequestType(self):
    """Wrong request type passed to remote method."""
    service = BasicService()

    self.assertRaises(remote.RequestError,
                      service.remote_method,
                      'wrong')

    self.assertRaises(remote.RequestError,
                      service.remote_method,
                      None)

    self.assertRaises(remote.RequestError,
                      service.remote_method,
                      SimpleResponse())

  def testInvocation_WrongResponseType(self):
    """Wrong response type returned from remote method."""

    class AnotherService(object):

      @remote.method(SimpleRequest, SimpleResponse)
      def remote_method(self, unused_request):
        return self.return_this

    service = AnotherService()

    service.return_this = 'wrong'
    self.assertRaises(remote.ServerError,
                      service.remote_method,
                      SimpleRequest())
    service.return_this = None
    self.assertRaises(remote.ServerError,
                      service.remote_method,
                      SimpleRequest())
    service.return_this = SimpleRequest()
    self.assertRaises(remote.ServerError,
                      service.remote_method,
                      SimpleRequest())

  def testBadRequestType(self):
    """Test bad request types used in remote definition."""

    for request_type in (None, 1020, messages.Message, str):

      def declare():
        class BadService(object):

          @remote.method(request_type, SimpleResponse)
          def remote_method(self, request):
            pass

      self.assertRaises(TypeError, declare)

  def testBadResponseType(self):
    """Test bad response types used in remote definition."""

    for response_type in (None, 1020, messages.Message, str):

      def declare():
        class BadService(object):

          @remote.method(SimpleRequest, response_type)
          def remote_method(self, request):
            pass

      self.assertRaises(TypeError, declare)

  def testDocString(self):
    """Test that the docstring comes from the original method."""
    service = BasicService()
    self.assertEquals('BasicService remote_method docstring.',
                      service.remote_method.__doc__)


class GetRemoteMethodTest(test_util.TestCase):
  """Test for is_remote_method."""

  def testGetRemoteMethod(self):
    """Test valid remote method detection."""

    class Service(object):

      @remote.method(Request, Response)
      def remote_method(self, request):
        pass

    self.assertEquals(Service.remote_method.remote,
                      remote.get_remote_method_info(Service.remote_method))
    self.assertTrue(Service.remote_method.remote,
                    remote.get_remote_method_info(Service().remote_method))

  def testGetNotRemoteMethod(self):
    """Test positive result on a remote method."""

    class NotService(object):

      def not_remote_method(self, request):
        pass

    def fn(self):
      pass

    class NotReallyRemote(object):
      """Test negative result on many bad values for remote methods."""

      def not_really(self, request):
        pass

      not_really.remote = 'something else'

    for not_remote in [NotService.not_remote_method,
                       NotService().not_remote_method,
                       NotReallyRemote.not_really,
                       NotReallyRemote().not_really,
                       None,
                       1,
                       'a string',
                       fn]:
      self.assertEquals(None, remote.get_remote_method_info(not_remote))


class RequestStateTest(test_util.TestCase):
  """Test request state."""

  STATE_CLASS = remote.RequestState

  def testConstructor(self):
    """Test constructor."""
    state = self.STATE_CLASS(remote_host='remote-host',
                             remote_address='remote-address',
                             server_host='server-host',
                             server_port=10)
    self.assertEquals('remote-host', state.remote_host)
    self.assertEquals('remote-address', state.remote_address)
    self.assertEquals('server-host', state.server_host)
    self.assertEquals(10, state.server_port)

    state = self.STATE_CLASS()
    self.assertEquals(None, state.remote_host)
    self.assertEquals(None, state.remote_address)
    self.assertEquals(None, state.server_host)
    self.assertEquals(None, state.server_port)

  def testConstructorError(self):
    """Test unexpected keyword argument."""
    self.assertRaises(TypeError,
                      self.STATE_CLASS,
                      x=10)

  def testRepr(self):
    """Test string representation."""
    self.assertEquals('<%s>' % self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS()))
    self.assertEquals("<%s remote_host='abc'>" % self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS(remote_host='abc')))
    self.assertEquals("<%s remote_host='abc' "
                      "remote_address='def'>" % self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS(remote_host='abc',
                                               remote_address='def')))
    self.assertEquals("<%s remote_host='abc' "
                      "remote_address='def' "
                      "server_host='ghi'>" % self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS(remote_host='abc',
                                            remote_address='def',
                                            server_host='ghi')))
    self.assertEquals("<%s remote_host='abc' "
                      "remote_address='def' "
                      "server_host='ghi' "
                      'server_port=102>' % self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS(remote_host='abc',
                                            remote_address='def',
                                            server_host='ghi',
                                            server_port=102)))


class HttpRequestStateTest(RequestStateTest):

  STATE_CLASS = remote.HttpRequestState

  def testHttpMethod(self):
    state = remote.HttpRequestState(http_method='GET')
    self.assertEquals('GET', state.http_method)

  def testHttpMethod(self):
    state = remote.HttpRequestState(service_path='/bar')
    self.assertEquals('/bar', state.service_path)

  def testHeadersList(self):
    state = remote.HttpRequestState(
      headers=[('a', 'b'), ('c', 'd'), ('c', 'e')])

    self.assertEquals(['a', 'c', 'c'], list(state.headers.keys()))
    self.assertEquals(['b'], state.headers.get_all('a'))
    self.assertEquals(['d', 'e'], state.headers.get_all('c'))

  def testHeadersDict(self):
    state = remote.HttpRequestState(headers={'a': 'b', 'c': ['d', 'e']})

    self.assertEquals(['a', 'c', 'c'], sorted(state.headers.keys()))
    self.assertEquals(['b'], state.headers.get_all('a'))
    self.assertEquals(['d', 'e'], state.headers.get_all('c'))

  def testRepr(self):
    super(HttpRequestStateTest, self).testRepr()

    self.assertEquals("<%s remote_host='abc' "
                      "remote_address='def' "
                      "server_host='ghi' "
                      'server_port=102 '
                      "http_method='POST' "
                      "service_path='/bar' "
                      "headers=[('a', 'b'), ('c', 'd')]>" %
                      self.STATE_CLASS.__name__,
                      repr(self.STATE_CLASS(remote_host='abc',
                                            remote_address='def',
                                            server_host='ghi',
                                            server_port=102,
                                            http_method='POST',
                                            service_path='/bar',
                                            headers={'a': 'b', 'c': 'd'},
                                            )))


class ServiceTest(test_util.TestCase):
  """Test Service class."""

  def testServiceBase_AllRemoteMethods(self):
    """Test that service base class has no remote methods."""
    self.assertEquals({}, remote.Service.all_remote_methods())

  def testAllRemoteMethods(self):
    """Test all_remote_methods with properly Service subclass."""
    self.assertEquals({'remote_method': MyService.remote_method},
                      MyService.all_remote_methods())

  def testAllRemoteMethods_SubClass(self):
    """Test all_remote_methods on a sub-class of a service."""
    class SubClass(MyService):

      @remote.method(Request, Response)
      def sub_class_method(self, request):
        pass

    self.assertEquals({'remote_method': SubClass.remote_method,
                       'sub_class_method': SubClass.sub_class_method,
                      },
                      SubClass.all_remote_methods())

  def testOverrideMethod(self):
    """Test that trying to override a remote method with remote decorator."""
    class SubClass(MyService):

      def remote_method(self, request):
        response = super(SubClass, self).remote_method(request)
        response.value = '(%s)' % response.value
        return response

    self.assertEquals({'remote_method': SubClass.remote_method,
                      },
                      SubClass.all_remote_methods())

    instance = SubClass()
    self.assertEquals('(Hello)',
                      instance.remote_method(Request(value='Hello')).value)
    self.assertEquals(Request, SubClass.remote_method.remote.request_type)
    self.assertEquals(Response, SubClass.remote_method.remote.response_type)

  def testOverrideMethodWithRemote(self):
    """Test trying to override a remote method with remote decorator."""
    def do_override():
      class SubClass(MyService):

        @remote.method(Request, Response)
        def remote_method(self, request):
          pass

    self.assertRaisesWithRegexpMatch(remote.ServiceDefinitionError,
                                     'Do not use method decorator when '
                                     'overloading remote method remote_method '
                                     'on service SubClass',
                                     do_override)

  def testOverrideMethodWithInvalidValue(self):
    """Test trying to override a remote method with remote decorator."""
    def do_override(bad_value):
      class SubClass(MyService):

        remote_method = bad_value

    for bad_value in [None, 1, 'string', {}]:
      self.assertRaisesWithRegexpMatch(remote.ServiceDefinitionError,
                                       'Must override remote_method in '
                                       'SubClass with a method',
                                       do_override, bad_value)

  def testCallingRemoteMethod(self):
    """Test invoking a remote method."""
    expected = Response()
    expected.value = 'what was passed in'

    request = Request()
    request.value = 'what was passed in'

    service = MyService()
    self.assertEquals(expected, service.remote_method(request))

  def testFactory(self):
    """Test using factory to pass in state."""
    class StatefulService(remote.Service):

      def __init__(self, a, b, c=None):
        self.a = a
        self.b = b
        self.c = c

    state = [1, 2, 3]

    factory = StatefulService.new_factory(1, state)

    module_name = ServiceTest.__module__
    pattern = ('Creates new instances of service StatefulService.\n\n'
               'Returns:\n'
               '  New instance of %s.StatefulService.' % module_name)
    self.assertEqual(pattern, factory.__doc__)
    self.assertEquals('StatefulService_service_factory', factory.__name__)
    self.assertEquals(StatefulService, factory.service_class)

    service = factory()
    self.assertEquals(1, service.a)
    self.assertEquals(id(state), id(service.b))
    self.assertEquals(None, service.c)

    factory = StatefulService.new_factory(2, b=3, c=4)
    service = factory()
    self.assertEquals(2, service.a)
    self.assertEquals(3, service.b)
    self.assertEquals(4, service.c)

  def testFactoryError(self):
    """Test misusing a factory."""
    # Passing positional argument that is not accepted by class.
    self.assertRaises(TypeError, remote.Service.new_factory(1))

    # Passing keyword argument that is not accepted by class.
    self.assertRaises(TypeError, remote.Service.new_factory(x=1))

    class StatefulService(remote.Service):

      def __init__(self, a):
        pass

    # Missing required parameter.
    self.assertRaises(TypeError, StatefulService.new_factory())

  def testDefinitionName(self):
    """Test getting service definition name."""
    class TheService(remote.Service):
      pass

    module_name = test_util.get_module_name(ServiceTest)
    self.assertEqual(TheService.definition_name(),
                     '%s.TheService' % module_name)
    self.assertTrue(TheService.outer_definition_name(),
                    module_name)
    self.assertTrue(TheService.definition_package(),
                    module_name)

  def testDefinitionNameWithPackage(self):
    """Test getting service definition name when package defined."""
    global package
    package = 'my.package'
    try:
      class TheService(remote.Service):
        pass

      self.assertEquals('my.package.TheService', TheService.definition_name())
      self.assertEquals('my.package', TheService.outer_definition_name())
      self.assertEquals('my.package', TheService.definition_package())
    finally:
      del package

  def testDefinitionNameWithNoModule(self):
    """Test getting service definition name when package defined."""
    module = sys.modules[__name__]
    try:
      del sys.modules[__name__]
      class TheService(remote.Service):
        pass

      self.assertEquals('TheService', TheService.definition_name())
      self.assertEquals(None, TheService.outer_definition_name())
      self.assertEquals(None, TheService.definition_package())
    finally:
      sys.modules[__name__] = module


class StubTest(test_util.TestCase):

  def setUp(self):
    self.mox = mox.Mox()
    self.transport = self.mox.CreateMockAnything()

  def testDefinitionName(self):
    self.assertEquals(BasicService.definition_name(),
                      BasicService.Stub.definition_name())
    self.assertEquals(BasicService.outer_definition_name(),
                      BasicService.Stub.outer_definition_name())
    self.assertEquals(BasicService.definition_package(),
                      BasicService.Stub.definition_package())

  def testRemoteMethods(self):
    self.assertEquals(BasicService.all_remote_methods(),
                      BasicService.Stub.all_remote_methods())

  def testSync_WithRequest(self):
    stub = BasicService.Stub(self.transport)

    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    rpc = transport.Rpc(request)
    rpc.set_response(response)
    self.transport.send_rpc(BasicService.remote_method.remote,
                            request).AndReturn(rpc)

    self.mox.ReplayAll()

    self.assertEquals(SimpleResponse(), stub.remote_method(request))

    self.mox.VerifyAll()

  def testSync_WithKwargs(self):
    stub = BasicService.Stub(self.transport)


    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    rpc = transport.Rpc(request)
    rpc.set_response(response)
    self.transport.send_rpc(BasicService.remote_method.remote,
                            request).AndReturn(rpc)

    self.mox.ReplayAll()

    self.assertEquals(SimpleResponse(), stub.remote_method(param1='val1',
                                                           param2='val2'))

    self.mox.VerifyAll()

  def testAsync_WithRequest(self):
    stub = BasicService.Stub(self.transport)

    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    rpc = transport.Rpc(request)

    self.transport.send_rpc(BasicService.remote_method.remote,
                            request).AndReturn(rpc)

    self.mox.ReplayAll()

    self.assertEquals(rpc, stub.proto_async.remote_method(request))

    self.mox.VerifyAll()

  def testAsync_WithKwargs(self):
    stub = BasicService.Stub(self.transport)

    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    rpc = transport.Rpc(request)

    self.transport.send_rpc(BasicService.remote_method.remote,
                            request).AndReturn(rpc)

    self.mox.ReplayAll()

    self.assertEquals(rpc, stub.proto_async.remote_method(param1='val1',
                                                    param2='val2'))

    self.mox.VerifyAll()

  def testAsync_WithRequestAndKwargs(self):
    stub = BasicService.Stub(self.transport)

    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    self.mox.ReplayAll()

    self.assertRaisesWithRegexpMatch(
      TypeError,
      r'May not provide both args and kwargs',
      stub.proto_async.remote_method,
      request,
      param1='val1',
      param2='val2')

    self.mox.VerifyAll()

  def testAsync_WithTooManyPositionals(self):
    stub = BasicService.Stub(self.transport)

    request = SimpleRequest()
    request.param1 = 'val1'
    request.param2 = 'val2'
    response = SimpleResponse()

    self.mox.ReplayAll()

    self.assertRaisesWithRegexpMatch(
      TypeError,
      r'remote_method\(\) takes at most 2 positional arguments \(3 given\)',
      stub.proto_async.remote_method,
      request, 'another value')

    self.mox.VerifyAll()


class IsErrorStatusTest(test_util.TestCase):

  def testIsError(self):
    for state in (s for s in remote.RpcState if s > remote.RpcState.RUNNING):
      status = remote.RpcStatus(state=state)
      self.assertTrue(remote.is_error_status(status))

  def testIsNotError(self):
    for state in (s for s in remote.RpcState if s <= remote.RpcState.RUNNING):
      status = remote.RpcStatus(state=state)
      self.assertFalse(remote.is_error_status(status))

  def testStateNone(self):
    self.assertRaises(messages.ValidationError,
                      remote.is_error_status, remote.RpcStatus())


class CheckRpcStatusTest(test_util.TestCase):

  def testStateNone(self):
    self.assertRaises(messages.ValidationError,
                      remote.check_rpc_status, remote.RpcStatus())

  def testNoError(self):
    for state in (remote.RpcState.OK, remote.RpcState.RUNNING):
      remote.check_rpc_status(remote.RpcStatus(state=state))

  def testErrorState(self):
    status = remote.RpcStatus(state=remote.RpcState.REQUEST_ERROR,
                              error_message='a request error')
    self.assertRaisesWithRegexpMatch(remote.RequestError,
                                     'a request error',
                                     remote.check_rpc_status, status)

  def testApplicationErrorState(self):
    status = remote.RpcStatus(state=remote.RpcState.APPLICATION_ERROR,
                              error_message='an application error',
                              error_name='blam')
    try:
      remote.check_rpc_status(status)
      self.fail('Should have raised application error.')
    except remote.ApplicationError as err:
      self.assertEquals('an application error', str(err))
      self.assertEquals('blam', err.error_name)


class ProtocolConfigTest(test_util.TestCase):

  def testConstructor(self):
    config = remote.ProtocolConfig(
      protojson,
      'proto1',
      'application/X-Json',
      iter(['text/Json', 'text/JavaScript']))
    self.assertEquals(protojson, config.protocol)
    self.assertEquals('proto1', config.name)
    self.assertEquals('application/x-json', config.default_content_type)
    self.assertEquals(('text/json', 'text/javascript'),
                      config.alternate_content_types)
    self.assertEquals(('application/x-json', 'text/json', 'text/javascript'),
                      config.content_types)

  def testConstructorDefaults(self):
    config = remote.ProtocolConfig(protojson, 'proto2')
    self.assertEquals(protojson, config.protocol)
    self.assertEquals('proto2', config.name)
    self.assertEquals('application/json', config.default_content_type)
    self.assertEquals(('application/x-javascript',
                       'text/javascript',
                       'text/x-javascript',
                       'text/x-json',
                       'text/json'),
                      config.alternate_content_types)
    self.assertEquals(('application/json',
                       'application/x-javascript',
                       'text/javascript',
                       'text/x-javascript',
                       'text/x-json',
                       'text/json'), config.content_types)

  def testEmptyAlternativeTypes(self):
    config = remote.ProtocolConfig(protojson, 'proto2',
                                   alternative_content_types=())
    self.assertEquals(protojson, config.protocol)
    self.assertEquals('proto2', config.name)
    self.assertEquals('application/json', config.default_content_type)
    self.assertEquals((), config.alternate_content_types)
    self.assertEquals(('application/json',), config.content_types)

  def testDuplicateContentTypes(self):
    self.assertRaises(remote.ServiceConfigurationError,
                      remote.ProtocolConfig,
                      protojson,
                      'json',
                      'text/plain',
                      ('text/plain',))

    self.assertRaises(remote.ServiceConfigurationError,
                      remote.ProtocolConfig,
                      protojson,
                      'json',
                      'text/plain',
                      ('text/html', 'text/html'))

  def testEncodeMessage(self):
    config = remote.ProtocolConfig(protojson, 'proto2')
    encoded_message = config.encode_message(
        remote.RpcStatus(state=remote.RpcState.SERVER_ERROR,
                         error_message='bad error'))

    # Convert back to a dictionary from JSON.
    dict_message = protojson.json.loads(encoded_message)
    self.assertEquals({'state': 'SERVER_ERROR', 'error_message': 'bad error'},
                      dict_message)

  def testDecodeMessage(self):
    config = remote.ProtocolConfig(protojson, 'proto2')
    self.assertEquals(
      remote.RpcStatus(state=remote.RpcState.SERVER_ERROR,
                       error_message="bad error"),
      config.decode_message(
        remote.RpcStatus,
        '{"state": "SERVER_ERROR", "error_message": "bad error"}'))


class ProtocolsTest(test_util.TestCase):

  def setUp(self):
    self.protocols = remote.Protocols()

  def testEmpty(self):
    self.assertEquals((), self.protocols.names)
    self.assertEquals((), self.protocols.content_types)

  def testAddProtocolAllDefaults(self):
    self.protocols.add_protocol(protojson, 'json')
    self.assertEquals(('json',), self.protocols.names)
    self.assertEquals(('application/json',
                       'application/x-javascript',
                       'text/javascript',
                       'text/json',
                       'text/x-javascript',
                       'text/x-json'),
                      self.protocols.content_types)

  def testAddProtocolNoDefaultAlternatives(self):
    class Protocol(object):
      CONTENT_TYPE = 'text/plain'

    self.protocols.add_protocol(Protocol, 'text')
    self.assertEquals(('text',), self.protocols.names)
    self.assertEquals(('text/plain',), self.protocols.content_types)

  def testAddProtocolOverrideDefaults(self):
    self.protocols.add_protocol(protojson, 'json',
                                default_content_type='text/blar',
                                alternative_content_types=('text/blam',
                                                           'text/blim'))
    self.assertEquals(('json',), self.protocols.names)
    self.assertEquals(('text/blam', 'text/blar', 'text/blim'),
                      self.protocols.content_types)

  def testLookupByName(self):
    self.protocols.add_protocol(protojson, 'json')
    self.protocols.add_protocol(protojson, 'json2',
                                default_content_type='text/plain',
                                alternative_content_types=())

    self.assertEquals('json', self.protocols.lookup_by_name('JsOn').name)
    self.assertEquals('json2', self.protocols.lookup_by_name('Json2').name)

  def testLookupByContentType(self):
    self.protocols.add_protocol(protojson, 'json')
    self.protocols.add_protocol(protojson, 'json2',
                                default_content_type='text/plain',
                                alternative_content_types=())

    self.assertEquals(
      'json',
      self.protocols.lookup_by_content_type('AppliCation/Json').name)

    self.assertEquals(
      'json',
      self.protocols.lookup_by_content_type('text/x-Json').name)

    self.assertEquals(
      'json2',
      self.protocols.lookup_by_content_type('text/Plain').name)

  def testNewDefault(self):
    protocols = remote.Protocols.new_default()
    self.assertEquals(('protobuf', 'protojson'), protocols.names)

    protobuf_protocol = protocols.lookup_by_name('protobuf')
    self.assertEquals(protobuf, protobuf_protocol.protocol)

    protojson_protocol = protocols.lookup_by_name('protojson')
    self.assertEquals(protojson.ProtoJson.get_default(),
                      protojson_protocol.protocol)

  def testGetDefaultProtocols(self):
    protocols = remote.Protocols.get_default()
    self.assertEquals(('protobuf', 'protojson'), protocols.names)

    protobuf_protocol = protocols.lookup_by_name('protobuf')
    self.assertEquals(protobuf, protobuf_protocol.protocol)

    protojson_protocol = protocols.lookup_by_name('protojson')
    self.assertEquals(protojson.ProtoJson.get_default(),
                      protojson_protocol.protocol)

    self.assertTrue(protocols is remote.Protocols.get_default())

  def testSetDefaultProtocols(self):
    protocols = remote.Protocols()
    remote.Protocols.set_default(protocols)
    self.assertTrue(protocols is remote.Protocols.get_default())

  def testSetDefaultWithoutProtocols(self):
    self.assertRaises(TypeError, remote.Protocols.set_default, None)
    self.assertRaises(TypeError, remote.Protocols.set_default, 'hi protocols')
    self.assertRaises(TypeError, remote.Protocols.set_default, {})


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Test utilities for message testing.

Includes module interface test to ensure that public parts of module are
correctly declared in __all__.

Includes message types that correspond to those defined in
services_test.proto.

Includes additional test utilities to make sure encoding/decoding libraries
conform.
"""
from six.moves import range

__author__ = 'rafek@google.com (Rafe Kaplan)'

import cgi
import datetime
import inspect
import os
import re
import socket
import types
import unittest2 as unittest

import six

from . import message_types
from . import messages
from . import util

# Unicode of the word "Russian" in cyrillic.
RUSSIAN = u'\u0440\u0443\u0441\u0441\u043a\u0438\u0439'

# All characters binary value interspersed with nulls.
BINARY = b''.join(six.int2byte(value) + b'\0' for value in range(256))


class TestCase(unittest.TestCase):

  def assertRaisesWithRegexpMatch(self,
                                  exception,
                                  regexp,
                                  function,
                                  *params,
                                  **kwargs):
    """Check that exception is raised and text matches regular expression.

    Args:
      exception: Exception type that is expected.
      regexp: String regular expression that is expected in error message.
      function: Callable to test.
      params: Parameters to forward to function.
      kwargs: Keyword arguments to forward to function.
    """
    try:
      function(*params, **kwargs)
      self.fail('Expected exception %s was not raised' % exception.__name__)
    except exception as err:
      match = bool(re.match(regexp, str(err)))
      self.assertTrue(match, 'Expected match "%s", found "%s"' % (regexp,
                                                                  err))

  def assertHeaderSame(self, header1, header2):
    """Check that two HTTP headers are the same.

    Args:
      header1: Header value string 1.
      header2: header value string 2.
    """
    value1, params1 = cgi.parse_header(header1)
    value2, params2 = cgi.parse_header(header2)
    self.assertEqual(value1, value2)
    self.assertEqual(params1, params2)

  def assertIterEqual(self, iter1, iter2):
    """Check that two iterators or iterables are equal independent of order.

    Similar to Python 2.7 assertItemsEqual.  Named differently in order to
    avoid potential conflict.

    Args:
      iter1: An iterator or iterable.
      iter2: An iterator or iterable.
    """
    list1 = list(iter1)
    list2 = list(iter2)

    unmatched1 = list()

    while list1:
      item1 = list1[0]
      del list1[0]
      for index in range(len(list2)):
        if item1 == list2[index]:
          del list2[index]
          break
      else:
        unmatched1.append(item1)

    error_message = []
    for item in unmatched1:
      error_message.append(
          '  Item from iter1 not found in iter2: %r' % item)
    for item in list2:
      error_message.append(
          '  Item from iter2 not found in iter1: %r' % item)
    if error_message:
      self.fail('Collections not equivalent:\n' + '\n'.join(error_message))


class ModuleInterfaceTest(object):
  """Test to ensure module interface is carefully constructed.

  A module interface is the set of public objects listed in the module __all__
  attribute.  Modules that that are considered public should have this interface
  carefully declared.  At all times, the __all__ attribute should have objects
  intended to be publically used and all other objects in the module should be
  considered unused.

  Protected attributes (those beginning with '_') and other imported modules
  should not be part of this set of variables.  An exception is for variables
  that begin and end with '__' which are implicitly part of the interface
  (eg. __name__, __file__, __all__ itself, etc.).

  Modules that are imported in to the tested modules are an exception and may
  be left out of the __all__ definition. The test is done by checking the value
  of what would otherwise be a public name and not allowing it to be exported
  if it is an instance of a module.  Modules that are explicitly exported are
  for the time being not permitted.

  To use this test class a module should define a new class that inherits first
  from ModuleInterfaceTest and then from test_util.TestCase.  No other tests
  should be added to this test case, making the order of inheritance less
  important, but if setUp for some reason is overidden, it is important that
  ModuleInterfaceTest is first in the list so that its setUp method is
  invoked.

  Multiple inheretance is required so that ModuleInterfaceTest is not itself
  a test, and is not itself executed as one.

  The test class is expected to have the following class attributes defined:

    MODULE: A reference to the module that is being validated for interface
      correctness.

  Example:
    Module definition (hello.py):

      import sys

      __all__ = ['hello']

      def _get_outputter():
        return sys.stdout

      def hello():
        _get_outputter().write('Hello\n')

    Test definition:

      import unittest
      from protorpc import test_util

      import hello

      class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                                test_util.TestCase):

        MODULE = hello


      class HelloTest(test_util.TestCase):
        ... Test 'hello' module ...


      if __name__ == '__main__':
        unittest.main()
  """

  def setUp(self):
    """Set up makes sure that MODULE and IMPORTED_MODULES is defined.

    This is a basic configuration test for the test itself so does not
    get it's own test case.
    """
    if not hasattr(self, 'MODULE'):
      self.fail(
          "You must define 'MODULE' on ModuleInterfaceTest sub-class %s." %
          type(self).__name__)

  def testAllExist(self):
    """Test that all attributes defined in __all__ exist."""
    missing_attributes = []
    for attribute in self.MODULE.__all__:
      if not hasattr(self.MODULE, attribute):
        missing_attributes.append(attribute)
    if missing_attributes:
      self.fail('%s of __all__ are not defined in module.' %
                missing_attributes)

  def testAllExported(self):
    """Test that all public attributes not imported are in __all__."""
    missing_attributes = []
    for attribute in dir(self.MODULE):
      if not attribute.startswith('_'):
        if (attribute not in self.MODULE.__all__ and
            not isinstance(getattr(self.MODULE, attribute),
                           types.ModuleType) and
            attribute != 'with_statement'):
          missing_attributes.append(attribute)
    if missing_attributes:
      self.fail('%s are not modules and not defined in __all__.' %
                missing_attributes)

  def testNoExportedProtectedVariables(self):
    """Test that there are no protected variables listed in __all__."""
    protected_variables = []
    for attribute in self.MODULE.__all__:
      if attribute.startswith('_'):
        protected_variables.append(attribute)
    if protected_variables:
      self.fail('%s are protected variables and may not be exported.' %
                protected_variables)

  def testNoExportedModules(self):
    """Test that no modules exist in __all__."""
    exported_modules = []
    for attribute in self.MODULE.__all__:
      try:
        value = getattr(self.MODULE, attribute)
      except AttributeError:
        # This is a different error case tested for in testAllExist.
        pass
      else:
        if isinstance(value, types.ModuleType):
          exported_modules.append(attribute)
    if exported_modules:
      self.fail('%s are modules and may not be exported.' % exported_modules)


class NestedMessage(messages.Message):
  """Simple message that gets nested in another message."""

  a_value = messages.StringField(1, required=True)


class HasNestedMessage(messages.Message):
  """Message that has another message nested in it."""

  nested = messages.MessageField(NestedMessage, 1)
  repeated_nested = messages.MessageField(NestedMessage, 2, repeated=True)


class HasDefault(messages.Message):
  """Has a default value."""

  a_value = messages.StringField(1, default=u'a default')


class OptionalMessage(messages.Message):
  """Contains all message types."""

  class SimpleEnum(messages.Enum):
    """Simple enumeration type."""
    VAL1 = 1
    VAL2 = 2

  double_value = messages.FloatField(1, variant=messages.Variant.DOUBLE)
  float_value = messages.FloatField(2, variant=messages.Variant.FLOAT)
  int64_value = messages.IntegerField(3, variant=messages.Variant.INT64)
  uint64_value = messages.IntegerField(4, variant=messages.Variant.UINT64)
  int32_value = messages.IntegerField(5, variant=messages.Variant.INT32)
  bool_value = messages.BooleanField(6, variant=messages.Variant.BOOL)
  string_value = messages.StringField(7, variant=messages.Variant.STRING)
  bytes_value = messages.BytesField(8, variant=messages.Variant.BYTES)
  enum_value = messages.EnumField(SimpleEnum, 10)

  # TODO(rafek): Add support for these variants.
  # uint32_value = messages.IntegerField(9, variant=messages.Variant.UINT32)
  # sint32_value = messages.IntegerField(11, variant=messages.Variant.SINT32)
  # sint64_value = messages.IntegerField(12, variant=messages.Variant.SINT64)


class RepeatedMessage(messages.Message):
  """Contains all message types as repeated fields."""

  class SimpleEnum(messages.Enum):
    """Simple enumeration type."""
    VAL1 = 1
    VAL2 = 2

  double_value = messages.FloatField(1,
                                     variant=messages.Variant.DOUBLE,
                                     repeated=True)
  float_value = messages.FloatField(2,
                                    variant=messages.Variant.FLOAT,
                                    repeated=True)
  int64_value = messages.IntegerField(3,
                                      variant=messages.Variant.INT64,
                                      repeated=True)
  uint64_value = messages.IntegerField(4,
                                       variant=messages.Variant.UINT64,
                                       repeated=True)
  int32_value = messages.IntegerField(5,
                                      variant=messages.Variant.INT32,
                                      repeated=True)
  bool_value = messages.BooleanField(6,
                                     variant=messages.Variant.BOOL,
                                     repeated=True)
  string_value = messages.StringField(7,
                                      variant=messages.Variant.STRING,
                                      repeated=True)
  bytes_value = messages.BytesField(8,
                                    variant=messages.Variant.BYTES,
                                    repeated=True)
  #uint32_value = messages.IntegerField(9, variant=messages.Variant.UINT32)
  enum_value = messages.EnumField(SimpleEnum,
                                  10,
                                  repeated=True)
  #sint32_value = messages.IntegerField(11, variant=messages.Variant.SINT32)
  #sint64_value = messages.IntegerField(12, variant=messages.Variant.SINT64)


class HasOptionalNestedMessage(messages.Message):

  nested = messages.MessageField(OptionalMessage, 1)
  repeated_nested = messages.MessageField(OptionalMessage, 2, repeated=True)


class ProtoConformanceTestBase(object):
  """Protocol conformance test base class.

  Each supported protocol should implement two methods that support encoding
  and decoding of Message objects in that format:

    encode_message(message) - Serialize to encoding.
    encode_message(message, encoded_message) - Deserialize from encoding.

  Tests for the modules where these functions are implemented should extend
  this class in order to support basic behavioral expectations.  This ensures
  that protocols correctly encode and decode message transparently to the
  caller.

  In order to support these test, the base class should also extend the TestCase
  class and implement the following class attributes which define the encoded
  version of certain protocol buffers:

    encoded_partial:
      <OptionalMessage
        double_value: 1.23
        int64_value: -100000000000
        string_value: u"a string"
        enum_value: OptionalMessage.SimpleEnum.VAL2
        >

    encoded_full:
      <OptionalMessage
        double_value: 1.23
        float_value: -2.5
        int64_value: -100000000000
        uint64_value: 102020202020
        int32_value: 1020
        bool_value: true
        string_value: u"a string\u044f"
        bytes_value: b"a bytes\xff\xfe"
        enum_value: OptionalMessage.SimpleEnum.VAL2
        >

    encoded_repeated:
      <RepeatedMessage
        double_value: [1.23, 2.3]
        float_value: [-2.5, 0.5]
        int64_value: [-100000000000, 20]
        uint64_value: [102020202020, 10]
        int32_value: [1020, 718]
        bool_value: [true, false]
        string_value: [u"a string\u044f", u"another string"]
        bytes_value: [b"a bytes\xff\xfe", b"another bytes"]
        enum_value: [OptionalMessage.SimpleEnum.VAL2,
                     OptionalMessage.SimpleEnum.VAL 1]
        >

    encoded_nested:
      <HasNestedMessage
        nested: <NestedMessage
          a_value: "a string"
          >
        >

    encoded_repeated_nested:
      <HasNestedMessage
        repeated_nested: [
            <NestedMessage a_value: "a string">,
            <NestedMessage a_value: "another string">
          ]
        >

    unexpected_tag_message:
      An encoded message that has an undefined tag or number in the stream.

    encoded_default_assigned:
      <HasDefault
        a_value: "a default"
        >

    encoded_nested_empty:
      <HasOptionalNestedMessage
        nested: <OptionalMessage>
        >

    encoded_invalid_enum:
      <OptionalMessage
        enum_value: (invalid value for serialization type)
        >
  """

  encoded_empty_message = ''

  def testEncodeInvalidMessage(self):
    message = NestedMessage()
    self.assertRaises(messages.ValidationError,
                      self.PROTOLIB.encode_message, message)

  def CompareEncoded(self, expected_encoded, actual_encoded):
    """Compare two encoded protocol values.

    Can be overridden by sub-classes to special case comparison.
    For example, to eliminate white space from output that is not
    relevant to encoding.

    Args:
      expected_encoded: Expected string encoded value.
      actual_encoded: Actual string encoded value.
    """
    self.assertEquals(expected_encoded, actual_encoded)

  def EncodeDecode(self, encoded, expected_message):
    message = self.PROTOLIB.decode_message(type(expected_message), encoded)
    self.assertEquals(expected_message, message)
    self.CompareEncoded(encoded, self.PROTOLIB.encode_message(message))

  def testEmptyMessage(self):
    self.EncodeDecode(self.encoded_empty_message, OptionalMessage())

  def testPartial(self):
    """Test message with a few values set."""
    message = OptionalMessage()
    message.double_value = 1.23
    message.int64_value = -100000000000
    message.int32_value = 1020
    message.string_value = u'a string'
    message.enum_value = OptionalMessage.SimpleEnum.VAL2

    self.EncodeDecode(self.encoded_partial, message)

  def testFull(self):
    """Test all types."""
    message = OptionalMessage()
    message.double_value = 1.23
    message.float_value = -2.5
    message.int64_value = -100000000000
    message.uint64_value = 102020202020
    message.int32_value = 1020
    message.bool_value = True
    message.string_value = u'a string\u044f'
    message.bytes_value = b'a bytes\xff\xfe'
    message.enum_value = OptionalMessage.SimpleEnum.VAL2

    self.EncodeDecode(self.encoded_full, message)

  def testRepeated(self):
    """Test repeated fields."""
    message = RepeatedMessage()
    message.double_value = [1.23, 2.3]
    message.float_value = [-2.5, 0.5]
    message.int64_value = [-100000000000, 20]
    message.uint64_value = [102020202020, 10]
    message.int32_value = [1020, 718]
    message.bool_value = [True, False]
    message.string_value = [u'a string\u044f', u'another string']
    message.bytes_value = [b'a bytes\xff\xfe', b'another bytes']
    message.enum_value = [RepeatedMessage.SimpleEnum.VAL2,
                          RepeatedMessage.SimpleEnum.VAL1]

    self.EncodeDecode(self.encoded_repeated, message)

  def testNested(self):
    """Test nested messages."""
    nested_message = NestedMessage()
    nested_message.a_value = u'a string'

    message = HasNestedMessage()
    message.nested = nested_message

    self.EncodeDecode(self.encoded_nested, message)

  def testRepeatedNested(self):
    """Test repeated nested messages."""
    nested_message1 = NestedMessage()
    nested_message1.a_value = u'a string'
    nested_message2 = NestedMessage()
    nested_message2.a_value = u'another string'

    message = HasNestedMessage()
    message.repeated_nested = [nested_message1, nested_message2]

    self.EncodeDecode(self.encoded_repeated_nested, message)

  def testStringTypes(self):
    """Test that encoding str on StringField works."""
    message = OptionalMessage()
    message.string_value = u'Latin'
    self.EncodeDecode(self.encoded_string_types, message)

  def testEncodeUninitialized(self):
    """Test that cannot encode uninitialized message."""
    required = NestedMessage()
    self.assertRaisesWithRegexpMatch(messages.ValidationError,
                                     "Message NestedMessage is missing "
                                     "required field a_value",
                                     self.PROTOLIB.encode_message,
                                     required)

  def testUnexpectedField(self):
    """Test decoding and encoding unexpected fields."""
    loaded_message = self.PROTOLIB.decode_message(OptionalMessage,
                                                  self.unexpected_tag_message)
    # Message should be equal to an empty message, since unknown values aren't
    # included in equality.
    self.assertEquals(OptionalMessage(), loaded_message)
    # Verify that the encoded message matches the source, including the
    # unknown value.
    self.assertEquals(self.unexpected_tag_message,
                      self.PROTOLIB.encode_message(loaded_message))

  def testDoNotSendDefault(self):
    """Test that default is not sent when nothing is assigned."""
    self.EncodeDecode(self.encoded_empty_message, HasDefault())

  def testSendDefaultExplicitlyAssigned(self):
    """Test that default is sent when explcitly assigned."""
    message = HasDefault()

    message.a_value = HasDefault.a_value.default

    self.EncodeDecode(self.encoded_default_assigned, message)

  def testEncodingNestedEmptyMessage(self):
    """Test encoding a nested empty message."""
    message = HasOptionalNestedMessage()
    message.nested = OptionalMessage()

    self.EncodeDecode(self.encoded_nested_empty, message)

  def testEncodingRepeatedNestedEmptyMessage(self):
    """Test encoding a nested empty message."""
    message = HasOptionalNestedMessage()
    message.repeated_nested = [OptionalMessage(), OptionalMessage()]

    self.EncodeDecode(self.encoded_repeated_nested_empty, message)

  def testContentType(self):
    self.assertTrue(isinstance(self.PROTOLIB.CONTENT_TYPE, str))

  def testDecodeInvalidEnumType(self):
    self.assertRaisesWithRegexpMatch(messages.DecodeError,
                                     'Invalid enum value ',
                                     self.PROTOLIB.decode_message,
                                     OptionalMessage,
                                     self.encoded_invalid_enum)

  def testDateTimeNoTimeZone(self):
    """Test that DateTimeFields are encoded/decoded correctly."""

    class MyMessage(messages.Message):
      value = message_types.DateTimeField(1)

    value = datetime.datetime(2013, 1, 3, 11, 36, 30, 123000)
    message = MyMessage(value=value)
    decoded = self.PROTOLIB.decode_message(
        MyMessage, self.PROTOLIB.encode_message(message))
    self.assertEquals(decoded.value, value)

  def testDateTimeWithTimeZone(self):
    """Test DateTimeFields with time zones."""

    class MyMessage(messages.Message):
      value = message_types.DateTimeField(1)

    value = datetime.datetime(2013, 1, 3, 11, 36, 30, 123000,
                              util.TimeZoneOffset(8 * 60))
    message = MyMessage(value=value)
    decoded = self.PROTOLIB.decode_message(
        MyMessage, self.PROTOLIB.encode_message(message))
    self.assertEquals(decoded.value, value)


def do_with(context, function, *args, **kwargs):
  """Simulate a with statement.

  Avoids need to import with from future.

  Does not support simulation of 'as'.

  Args:
    context: Context object normally used with 'with'.
    function: Callable to evoke.  Replaces with-block.
  """
  context.__enter__()
  try:
    function(*args, **kwargs)
  except:
    context.__exit__(*sys.exc_info())
  finally:
    context.__exit__(None, None, None)


def pick_unused_port():
  """Find an unused port to use in tests.

    Derived from Damon Kohlers example:

      http://code.activestate.com/recipes/531822-pick-unused-port
  """
  try:
    temp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
  except socket.error:
    # Try IPv6
    temp = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)

  try:
    temp.bind(('localhost', 0))
    port = temp.getsockname()[1]
  finally:
    temp.close()
  return port


def get_module_name(module_attribute):
  """Get the module name.

  Args:
    module_attribute: An attribute of the module.

  Returns:
    The fully qualified module name or simple module name where
    'module_attribute' is defined if the module name is "__main__".
  """
  if module_attribute.__module__ == '__main__':
    module_file = inspect.getfile(module_attribute)
    default = os.path.basename(module_file).split('.')[0]
    return default
  else:
    return module_attribute.__module__



#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Command line tool for generating ProtoRPC definitions from descriptors."""

import errno
import logging
import optparse
import os
import sys

from protorpc import descriptor
from protorpc import generate_python
from protorpc import protobuf
from protorpc import registry
from protorpc import transport
from protorpc import util

EXCLUDED_PACKAGES = frozenset(['protorpc.registry',
                               'protorpc.messages',
                               'protorpc.descriptor',
                               'protorpc.message_types',
                               ])

commands = {}


def usage():
  """Print usage help and exit with an error code."""
  parser.print_help()
  sys.exit(2)


def fatal_error(message):
  """Print fatal error messages exit with an error code.

  Args:
    message: Message to print to stderr before exit.
  """
  sys.stderr.write(message)
  sys.exit(1)


def open_input_file(filename):
  """Open file for reading.

  Args:
    filename: Name of input file to open or None to open stdin.

  Returns:
    Opened file if string provided, stdin if filename is None.
  """
  # TODO(rafek): Detect missing or invalid files, generating user friendly
  # error messages.
  if filename is None:
    return sys.stdin
  else:
    try:
      return open(filename, 'rb')
    except IOError, err:
      fatal_error(str(err))


@util.positional(1)
def generate_file_descriptor(dest_dir, file_descriptor, force_overwrite):
  """Generate a single file descriptor to destination directory.

  Will generate a single Python file from a file descriptor under dest_dir.
  The sub-directory where the file is generated is determined by the package
  name of descriptor.

  Descriptors without package names will not be generated.

  Descriptors that are part of the ProtoRPC distribution will not be generated.

  Args:
    dest_dir: Directory under which to generate files.
    file_descriptor: FileDescriptor instance to generate source code from.
    force_overwrite: If True, existing files will be overwritten.
  """
  package = file_descriptor.package
  if not package:
    # TODO(rafek): Option to cause an error on this condition.
    logging.warn('Will not generate descriptor without package name')
    return

  if package in EXCLUDED_PACKAGES:
    logging.warn('Will not generate main ProtoRPC class %s' % package)
    return

  package_path = package.split('.')
  directory = package_path[:-1]
  package_file_name = package_path[-1]
  directory_name = os.path.join(dest_dir, *directory)
  output_file_name = os.path.join(directory_name,
                                  '%s.py' % (package_file_name,))

  try:
    os.makedirs(directory_name)
  except OSError, err:
    if err.errno != errno.EEXIST:
      raise

  if not force_overwrite and os.path.exists(output_file_name):
    logging.warn('Not overwriting %s with package %s',
                 output_file_name, package)
    return

  output_file = open(output_file_name, 'w')

  logging.info('Writing package %s to %s',
               file_descriptor.package, output_file_name)
  generate_python.format_python_file(file_descriptor, output_file)


@util.positional(1)
def command(name, required=(), optional=()):
  """Decorator used for declaring commands used on command line.

  Each command of this tool can have any number of sequential required
  parameters and optional parameters.  The required and optional parameters
  will be displayed in the command usage.  Arguments passed in to the command
  are checked to ensure they have at least the required parameters and not
  too many parameters beyond the optional ones.  When there are not enough
  or too few parameters the usage message is generated and the program exits
  with an error code.

  Functions decorated thus are added to commands by their name.

  Resulting decorated functions will have required and optional attributes
  assigned to them so that appear in the usage message.

  Args:
    name: Name of command that will follow the program name on the command line.
    required: List of required parameter names as displayed in the usage
      message.
    optional: List of optional parameter names as displayed in the usage
      message.
  """
  def check_params_decorator(function):
    def check_params_wrapper(options, *args):
      if not (len(required) <= len(args) <= len(required) + len(optional)):
        sys.stderr.write("Incorrect usage for command '%s'\n\n" % name)
        usage()
      function(options, *args)
    check_params_wrapper.required = required
    check_params_wrapper.optional = optional
    commands[name] = check_params_wrapper
    return check_params_wrapper
  return check_params_decorator


@command('file', optional=['input-filename', 'output-filename'])
def file_command(options, input_filename=None, output_filename=None):
  """Generate a single descriptor file to Python.

  Args:
    options: Parsed command line options.
    input_filename: File to read protobuf FileDescriptor from.  If None
      will read from stdin.
    output_filename: File to write Python source code to.  If None will
      generate to stdout.
  """
  with open_input_file(input_filename) as input_file:
    descriptor_content = input_file.read()

  if output_filename:
    output_file = open(output_filename, 'w')
  else:
    output_file = sys.stdout

  file_descriptor = protobuf.decode_message(descriptor.FileDescriptor,
                                            descriptor_content)
  generate_python.format_python_file(file_descriptor, output_file)


@command('fileset', optional=['filename'])
def fileset_command(options, input_filename=None):
  """Generate source directory structure from FileSet.

  Args:
    options: Parsed command line options.
    input_filename: File to read protobuf FileSet from.  If None will read from
      stdin.
  """
  with open_input_file(input_filename) as input_file:
    descriptor_content = input_file.read()

  dest_dir = os.path.expanduser(options.dest_dir)

  if not os.path.isdir(dest_dir) and os.path.exists(dest_dir):
    fatal_error("Destination '%s' is not a directory" % dest_dir)

  file_set = protobuf.decode_message(descriptor.FileSet,
                                     descriptor_content)

  for file_descriptor in file_set.files:
    generate_file_descriptor(dest_dir, file_descriptor=file_descriptor,
                             force_overwrite=options.force)


@command('registry',
         required=['host'],
         optional=['service-name', 'registry-path'])
def registry_command(options,
                     host,
                     service_name=None,
                     registry_path='/protorpc'):
  """Generate source directory structure from remote registry service.

  Args:
    options: Parsed command line options.
    host: Web service host where registry service is located.  May include
      port.
    service_name: Name of specific service to read.  Will generate only Python
      files that service is dependent on.  If None, will generate source code
      for all services known by the registry.
    registry_path: Path to find registry if not the default 'protorpc'.
  """
  dest_dir = os.path.expanduser(options.dest_dir)

  url = 'http://%s%s' % (host, registry_path)
  reg = registry.RegistryService.Stub(transport.HttpTransport(url))

  if service_name is None:
    service_names = [service.name for service in reg.services().services]
  else:
    service_names = [service_name]

  file_set = reg.get_file_set(names=service_names).file_set

  for file_descriptor in file_set.files:
    generate_file_descriptor(dest_dir, file_descriptor=file_descriptor,
                             force_overwrite=options.force)


def make_opt_parser():
  """Create options parser with automatically generated command help.

  Will iterate over all functions in commands and generate an appropriate
  usage message for them with all their required and optional parameters.
  """
  command_descriptions = []
  for name in sorted(commands.iterkeys()):
    command = commands[name]
    params = ' '.join(['<%s>' % param for param in command.required] +
                      ['[<%s>]' % param for param in command.optional])
    command_descriptions.append('%%prog [options] %s %s' % (name, params))
  command_usage = 'usage: %s\n' % '\n       '.join(command_descriptions)

  parser = optparse.OptionParser(usage=command_usage)
  parser.add_option('-d', '--dest_dir',
                    dest='dest_dir',
                    default=os.getcwd(),
                    help='Write generated files to DIR',
                    metavar='DIR')
  parser.add_option('-f', '--force',
                    action='store_true',
                    dest='force',
                    default=False,
                    help='Force overwrite of existing files')
  return parser

parser = make_opt_parser()


def main():
  # TODO(rafek): Customize verbosity.
  logging.basicConfig(level=logging.INFO)
  options, positional = parser.parse_args()

  if not positional:
    usage()

  command_name = positional[0]
  command = commands.get(command_name)
  if not command:
    sys.stderr.write("Unknown command '%s'\n\n" % command_name)
    usage()
  parameters = positional[1:]

  command(options, *parameters)


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""ProtoRPC WSGI service applications.

Use functions in this module to configure ProtoRPC services for use with
WSGI applications.  For more information about WSGI, please see:

  http://wsgi.org/wsgi
  http://docs.python.org/library/wsgiref.html
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import cgi
import six.moves.http_client
import logging
import re

from .. import messages
from .. import registry
from .. import remote
from .. import util
from . import util as wsgi_util

__all__ = [
  'DEFAULT_REGISTRY_PATH',
  'service_app',
]

_METHOD_PATTERN = r'(?:\.([^?]+))'
_REQUEST_PATH_PATTERN = r'^(%%s)%s$' % _METHOD_PATTERN

_HTTP_BAD_REQUEST = wsgi_util.error(six.moves.http_client.BAD_REQUEST)
_HTTP_NOT_FOUND = wsgi_util.error(six.moves.http_client.NOT_FOUND)
_HTTP_UNSUPPORTED_MEDIA_TYPE = wsgi_util.error(six.moves.http_client.UNSUPPORTED_MEDIA_TYPE)

DEFAULT_REGISTRY_PATH = '/protorpc'


@util.positional(2)
def service_mapping(service_factory, service_path=r'.*', protocols=None):
  """WSGI application that handles a single ProtoRPC service mapping.

  Args:
    service_factory: Service factory for creating instances of service request
      handlers.  Either callable that takes no parameters and returns a service
      instance or a service class whose constructor requires no parameters.
    service_path: Regular expression for matching requests against.  Requests
      that do not have matching paths will cause a 404 (Not Found) response.
    protocols: remote.Protocols instance that configures supported protocols
      on server.
  """
  service_class = getattr(service_factory, 'service_class', service_factory)
  remote_methods = service_class.all_remote_methods()
  path_matcher = re.compile(_REQUEST_PATH_PATTERN % service_path)

  def protorpc_service_app(environ, start_response):
    """Actual WSGI application function."""
    path_match = path_matcher.match(environ['PATH_INFO'])
    if not path_match:
      return _HTTP_NOT_FOUND(environ, start_response)
    service_path = path_match.group(1)
    method_name = path_match.group(2)

    content_type = environ.get('CONTENT_TYPE')
    if not content_type:
      content_type = environ.get('HTTP_CONTENT_TYPE')
    if not content_type:
      return _HTTP_BAD_REQUEST(environ, start_response)

    # TODO(rafek): Handle alternate encodings.
    content_type = cgi.parse_header(content_type)[0]

    request_method = environ['REQUEST_METHOD']
    if request_method != 'POST':
      content = ('%s.%s is a ProtoRPC method.\n\n'
                 'Service %s\n\n'
                 'More about ProtoRPC: '
                 '%s\n' %
                 (service_path,
                  method_name,
                  service_class.definition_name().encode('utf-8'),
                  util.PROTORPC_PROJECT_URL))
      error_handler = wsgi_util.error(
        six.moves.http_client.METHOD_NOT_ALLOWED,
        six.moves.http_client.responses[six.moves.http_client.METHOD_NOT_ALLOWED],
        content=content,
        content_type='text/plain; charset=utf-8')
      return error_handler(environ, start_response)

    local_protocols = protocols or remote.Protocols.get_default()
    try:
      protocol = local_protocols.lookup_by_content_type(content_type)
    except KeyError:
      return _HTTP_UNSUPPORTED_MEDIA_TYPE(environ,start_response)

    def send_rpc_error(status_code, state, message, error_name=None):
      """Helper function to send an RpcStatus message as response.

      Will create static error handler and begin response.

      Args:
        status_code: HTTP integer status code.
        state: remote.RpcState enum value to send as response.
        message: Helpful message to send in response.
        error_name: Error name if applicable.

      Returns:
        List containing encoded content response using the same content-type as
        the request.
      """
      status = remote.RpcStatus(state=state,
                                error_message=message,
                                error_name=error_name)
      encoded_status = protocol.encode_message(status)
      error_handler = wsgi_util.error(
        status_code,
        content_type=protocol.default_content_type,
        content=encoded_status)
      return error_handler(environ, start_response)

    method = remote_methods.get(method_name)
    if not method:
      return send_rpc_error(six.moves.http_client.BAD_REQUEST,
                            remote.RpcState.METHOD_NOT_FOUND_ERROR,
                            'Unrecognized RPC method: %s' % method_name)

    content_length = int(environ.get('CONTENT_LENGTH') or '0')

    remote_info = method.remote
    try:
      request = protocol.decode_message(
        remote_info.request_type, environ['wsgi.input'].read(content_length))
    except (messages.ValidationError, messages.DecodeError) as err:
      return send_rpc_error(six.moves.http_client.BAD_REQUEST,
                            remote.RpcState.REQUEST_ERROR,
                            'Error parsing ProtoRPC request '
                            '(Unable to parse request content: %s)' % err)

    instance = service_factory()

    initialize_request_state = getattr(
      instance, 'initialize_request_state', None)
    if initialize_request_state:
      # TODO(rafek): This is not currently covered by tests.
      server_port = environ.get('SERVER_PORT', None)
      if server_port:
        server_port = int(server_port)

      headers = []
      for name, value in six.iteritems(environ):
        if name.startswith('HTTP_'):
          headers.append((name[len('HTTP_'):].lower().replace('_', '-'), value))
      request_state = remote.HttpRequestState(
        remote_host=environ.get('REMOTE_HOST', None),
        remote_address=environ.get('REMOTE_ADDR', None),
        server_host=environ.get('SERVER_HOST', None),
        server_port=server_port,
        http_method=request_method,
        service_path=service_path,
        headers=headers)

      initialize_request_state(request_state)

    try:
      response = method(instance, request)
      encoded_response = protocol.encode_message(response)
    except remote.ApplicationError as err:
      return send_rpc_error(six.moves.http_client.BAD_REQUEST,
                            remote.RpcState.APPLICATION_ERROR,
                            str(err),
                            err.error_name)
    except Exception as err:
      logging.exception('Encountered unexpected error from ProtoRPC '
                        'method implementation: %s (%s)' %
                        (err.__class__.__name__, err))
      return send_rpc_error(six.moves.http_client.INTERNAL_SERVER_ERROR,
                            remote.RpcState.SERVER_ERROR,
                            'Internal Server Error')

    response_headers = [('content-type', content_type)]
    start_response('%d %s' % (six.moves.http_client.OK, six.moves.http_client.responses[six.moves.http_client.OK],),
                   response_headers)
    return [encoded_response]

  # Return WSGI application.
  return protorpc_service_app


@util.positional(1)
def service_mappings(services, registry_path=DEFAULT_REGISTRY_PATH):
  """Create multiple service mappings with optional RegistryService.

  Use this function to create single WSGI application that maps to
  multiple ProtoRPC services plus an optional RegistryService.

  Example:
    services = service.service_mappings(
        [(r'/time', TimeService),
         (r'/weather', WeatherService)
        ])

    In this example, the services WSGI application will map to two services,
    TimeService and WeatherService to the '/time' and '/weather' paths
    respectively.  In addition, it will also add a ProtoRPC RegistryService
    configured to serve information about both services at the (default) path
    '/protorpc'.

  Args:
    services: If a dictionary is provided instead of a list of tuples, the
      dictionary item pairs are used as the mappings instead.
      Otherwise, a list of tuples (service_path, service_factory):
      service_path: The path to mount service on.
      service_factory: A service class or service instance factory.
    registry_path: A string to change where the registry is mapped (the default
      location is '/protorpc').  When None, no registry is created or mounted.

  Returns:
    WSGI application that serves ProtoRPC services on their respective URLs
    plus optional RegistryService.
  """
  if isinstance(services, dict):
    services = six.iteritems(services)

  final_mapping = []
  paths = set()
  registry_map = {} if registry_path else None

  for service_path, service_factory in services:
    try:
      service_class = service_factory.service_class
    except AttributeError:
      service_class = service_factory

    if service_path not in paths:
      paths.add(service_path)
    else:
      raise remote.ServiceConfigurationError(
        'Path %r is already defined in service mapping' %
        service_path.encode('utf-8'))

    if registry_map is not None:
      registry_map[service_path] = service_class

    final_mapping.append(service_mapping(service_factory, service_path))

  if registry_map is not None:
    final_mapping.append(service_mapping(
      registry.RegistryService.new_factory(registry_map), registry_path))

  return wsgi_util.first_found(final_mapping)



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.generate_python_test."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import os
import shutil
import sys
import tempfile
import unittest

from protorpc import descriptor
from protorpc import generate_python
from protorpc import test_util
from protorpc import util


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = generate_python


class FormatPythonFileTest(test_util.TestCase):

  def setUp(self):
    self.original_path = list(sys.path)
    self.original_modules = dict(sys.modules)
    sys.path = list(sys.path)
    self.file_descriptor = descriptor.FileDescriptor()

    # Create temporary directory and add to Python path so that generated
    # Python code can be easily parsed, imported and executed.
    self.temp_dir = tempfile.mkdtemp()
    sys.path.append(self.temp_dir)

  def tearDown(self):
    # Reset path.
    sys.path[:] = []
    sys.path.extend(self.original_path)

    # Reset modules.
    sys.modules.clear()
    sys.modules.update(self.original_modules)

    # Remove temporary directory.
    try:
      shutil.rmtree(self.temp_dir)
    except IOError:
      pass

  def DoPythonTest(self, file_descriptor):
    """Execute python test based on a FileDescriptor object.

    The full test of the Python code generation is to generate a Python source
    code file, import the module and regenerate the FileDescriptor from it.
    If the generated FileDescriptor is the same as the original, it means that
    the generated source code correctly implements the actual FileDescriptor.
    """
    file_name = os.path.join(self.temp_dir,
                             '%s.py' % (file_descriptor.package or 'blank',))
    source_file = open(file_name, 'wt')
    try:
      generate_python.format_python_file(file_descriptor, source_file)
    finally:
      source_file.close()

    module_to_import = file_descriptor.package or 'blank'
    module = __import__(module_to_import)

    if not file_descriptor.package:
      self.assertFalse(hasattr(module, 'package'))
      module.package = ''  # Create package name so that comparison will work.

    reloaded_descriptor = descriptor.describe_file(module)

    # Need to sort both message_types fields because document order is never
    # Ensured.
    # TODO(rafek): Ensure document order.
    if reloaded_descriptor.message_types:
      reloaded_descriptor.message_types = sorted(
        reloaded_descriptor.message_types, key=lambda v: v.name)

    if file_descriptor.message_types:
      file_descriptor.message_types = sorted(
        file_descriptor.message_types, key=lambda v: v.name)

    self.assertEquals(file_descriptor, reloaded_descriptor)

  @util.positional(2)
  def DoMessageTest(self,
                    field_descriptors,
                    message_types=None,
                    enum_types=None):
    """Execute message generation test based on FieldDescriptor objects.

    Args:
      field_descriptor: List of FieldDescriptor object to generate and test.
      message_types: List of other MessageDescriptor objects that the new
        Message class depends on.
      enum_types: List of EnumDescriptor objects that the new Message class
        depends on.
    """
    file_descriptor = descriptor.FileDescriptor()
    file_descriptor.package = 'my_package'

    message_descriptor = descriptor.MessageDescriptor()
    message_descriptor.name = 'MyMessage'

    message_descriptor.fields = list(field_descriptors)

    file_descriptor.message_types = message_types or []
    file_descriptor.message_types.append(message_descriptor)

    if enum_types:
      file_descriptor.enum_types = list(enum_types)

    self.DoPythonTest(file_descriptor)

  def testBlankPackage(self):
    self.DoPythonTest(descriptor.FileDescriptor())

  def testEmptyPackage(self):
    file_descriptor = descriptor.FileDescriptor()
    file_descriptor.package = 'mypackage'
    self.DoPythonTest(file_descriptor)

  def testSingleField(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.INT64

    self.DoMessageTest([field])

  def testMessageField_InternalReference(self):
    other_message = descriptor.MessageDescriptor()
    other_message.name = 'OtherMessage'

    field = descriptor.FieldDescriptor()
    field.name = 'message_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.MESSAGE
    field.type_name = 'my_package.OtherMessage'

    self.DoMessageTest([field], message_types=[other_message])

  def testMessageField_ExternalReference(self):
    field = descriptor.FieldDescriptor()
    field.name = 'message_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.MESSAGE
    field.type_name = 'protorpc.registry.GetFileSetResponse'

    self.DoMessageTest([field])

  def testEnumField_InternalReference(self):
    enum = descriptor.EnumDescriptor()
    enum.name = 'Color'

    field = descriptor.FieldDescriptor()
    field.name = 'color'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.ENUM
    field.type_name = 'my_package.Color'

    self.DoMessageTest([field], enum_types=[enum])

  def testEnumField_ExternalReference(self):
    field = descriptor.FieldDescriptor()
    field.name = 'color'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.ENUM
    field.type_name = 'protorpc.descriptor.FieldDescriptor.Label'

    self.DoMessageTest([field])

  def testDateTimeField(self):
    field = descriptor.FieldDescriptor()
    field.name = 'timestamp'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.MESSAGE
    field.type_name = 'protorpc.message_types.DateTimeMessage'

    self.DoMessageTest([field])

  def testNonDefaultVariant(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.UINT64

    self.DoMessageTest([field])

  def testRequiredField(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.REQUIRED
    field.variant = descriptor.FieldDescriptor.Variant.INT64

    self.DoMessageTest([field])

  def testRepeatedField(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.REPEATED
    field.variant = descriptor.FieldDescriptor.Variant.INT64

    self.DoMessageTest([field])

  def testIntegerDefaultValue(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.INT64
    field.default_value = '10'

    self.DoMessageTest([field])

  def testFloatDefaultValue(self):
    field = descriptor.FieldDescriptor()
    field.name = 'float_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.DOUBLE
    field.default_value = '10.1'

    self.DoMessageTest([field])

  def testStringDefaultValue(self):
    field = descriptor.FieldDescriptor()
    field.name = 'string_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.STRING
    field.default_value = u'a nice lovely string\'s "string"'

    self.DoMessageTest([field])

  def testEnumDefaultValue(self):
    field = descriptor.FieldDescriptor()
    field.name = 'label'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.ENUM
    field.type_name = 'protorpc.descriptor.FieldDescriptor.Label'
    field.default_value = '2'

    self.DoMessageTest([field])

  def testMultiFields(self):
    field1 = descriptor.FieldDescriptor()
    field1.name = 'integer_field'
    field1.number = 1
    field1.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field1.variant = descriptor.FieldDescriptor.Variant.INT64

    field2 = descriptor.FieldDescriptor()
    field2.name = 'string_field'
    field2.number = 2
    field2.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field2.variant = descriptor.FieldDescriptor.Variant.STRING

    field3 = descriptor.FieldDescriptor()
    field3.name = 'unsigned_integer_field'
    field3.number = 3
    field3.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field3.variant = descriptor.FieldDescriptor.Variant.UINT64

    self.DoMessageTest([field1, field2, field3])

  def testNestedMessage(self):
    message = descriptor.MessageDescriptor()
    message.name = 'OuterMessage'

    inner_message = descriptor.MessageDescriptor()
    inner_message.name = 'InnerMessage'

    inner_inner_message = descriptor.MessageDescriptor()
    inner_inner_message.name = 'InnerInnerMessage'

    inner_message.message_types = [inner_inner_message]

    message.message_types = [inner_message]

    file_descriptor = descriptor.FileDescriptor()
    file_descriptor.message_types = [message]

    self.DoPythonTest(file_descriptor)

  def testNestedEnum(self):
    message = descriptor.MessageDescriptor()
    message.name = 'OuterMessage'

    inner_enum = descriptor.EnumDescriptor()
    inner_enum.name = 'InnerEnum'

    message.enum_types = [inner_enum]

    file_descriptor = descriptor.FileDescriptor()
    file_descriptor.message_types = [message]

    self.DoPythonTest(file_descriptor)

  def testService(self):
    service = descriptor.ServiceDescriptor()
    service.name = 'TheService'

    method1 = descriptor.MethodDescriptor()
    method1.name = 'method1'
    method1.request_type = 'protorpc.descriptor.FileDescriptor'
    method1.response_type = 'protorpc.descriptor.MethodDescriptor'

    service.methods = [method1]

    file_descriptor = descriptor.FileDescriptor()
    file_descriptor.service_types = [service]

    self.DoPythonTest(file_descriptor)

    # Test to make sure that implementation methods raise an exception.
    import blank
    service_instance = blank.TheService()
    self.assertRaisesWithRegexpMatch(NotImplementedError,
                                     'Method method1 is not implemented',
                                     service_instance.method1,
                                     descriptor.FileDescriptor())
  

def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

__author__ = 'rafek@google.com (Rafe Kaplan)'

import contextlib

from . import messages
from . import util

__all__ = ['IndentationError',
           'IndentWriter',
          ]


class IndentationError(messages.Error):
  """Raised when end_indent is called too many times."""


class IndentWriter(object):  
  """Utility class to make it easy to write formatted indented text.

  IndentWriter delegates to a file-like object and is able to keep track of the
  level of indentation.  Each call to write_line will write a line terminated
  by a new line proceeded by a number of spaces indicated by the current level
  of indentation.

  IndexWriter overloads the << operator to make line writing operations clearer.

  The indent method returns a context manager that can be used by the Python
  with statement that makes generating python code easier to use.  For example:

    index_writer << 'def factorial(n):'
    with index_writer.indent():
      index_writer << 'if n <= 1:'
      with index_writer.indent():
        index_writer << 'return 1'
      index_writer << 'else:'
      with index_writer.indent():
        index_writer << 'return factorial(n - 1)'

  This would generate:

  def factorial(n):
    if n <= 1:
      return 1
    else:
      return factorial(n - 1)
  """

  @util.positional(2)
  def __init__(self, output, indent_space=2):
    """Constructor.

    Args:
      output: File-like object to wrap.
      indent_space: Number of spaces each level of indentation will be.
    """
    # Private attributes:
    #
    #   __output: The wrapped file-like object.
    #   __indent_space: String to append for each level of indentation.
    #   __indentation: The current full indentation string.
    self.__output = output
    self.__indent_space = indent_space * ' '
    self.__indentation = 0

  @property
  def indent_level(self):
    """Current level of indentation for IndentWriter."""
    return self.__indentation

  def write_line(self, line):
    """Write line to wrapped file-like object using correct indentation.

    The line is written with the current level of indentation printed before it
    and terminated by a new line.

    Args:
      line: Line to write to wrapped file-like object.
    """
    if line != '':
      self.__output.write(self.__indentation * self.__indent_space)
      self.__output.write(line)
    self.__output.write('\n')

  def begin_indent(self):
    """Begin a level of indentation."""
    self.__indentation += 1

  def end_indent(self):
    """Undo the most recent level of indentation.

    Raises:
      IndentationError when called with no indentation levels.
    """
    if not self.__indentation:
      raise IndentationError('Unable to un-indent further')
    self.__indentation -= 1

  @contextlib.contextmanager
  def indent(self):
    """Create indentation level compatible with the Python 'with' keyword."""
    self.begin_indent()
    yield
    self.end_indent()

  def __lshift__(self, line):
    """Syntactic sugar for write_line method.

    Args:
      line: Line to write to wrapped file-like object.
    """
    self.write_line(line)



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.generate_proto_test."""


import os
import shutil
import cStringIO
import sys
import tempfile
import unittest

from protorpc import descriptor
from protorpc import generate_proto
from protorpc import test_util
from protorpc import util


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = generate_proto


class FormatProtoFileTest(test_util.TestCase):

  def setUp(self):
    self.file_descriptor = descriptor.FileDescriptor()
    self.output = cStringIO.StringIO()

  @property
  def result(self):
    return self.output.getvalue()

  def MakeMessage(self, name='MyMessage', fields=[]):
    message = descriptor.MessageDescriptor()
    message.name = name
    message.fields = fields

    messages_list = getattr(self.file_descriptor, 'fields', [])
    messages_list.append(message)
    self.file_descriptor.message_types = messages_list

  def testBlankPackage(self):
    self.file_descriptor.package = None
    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('', self.result)

  def testEmptyPackage(self):
    self.file_descriptor.package = 'my_package'
    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('package my_package;\n', self.result)

  def testSingleField(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.INT64

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      '  optional int64 integer_field = 1;\n'
                      '}\n',
                      self.result)

  def testSingleFieldWithDefault(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.INT64
    field.default_value = '10'

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      '  optional int64 integer_field = 1 [default=10];\n'
                      '}\n',
                      self.result)

  def testRepeatedFieldWithDefault(self):
    field = descriptor.FieldDescriptor()
    field.name = 'integer_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.REPEATED
    field.variant = descriptor.FieldDescriptor.Variant.INT64
    field.default_value = '[10, 20]'

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      '  repeated int64 integer_field = 1;\n'
                      '}\n',
                      self.result)

  def testSingleFieldWithDefaultString(self):
    field = descriptor.FieldDescriptor()
    field.name = 'string_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.STRING
    field.default_value = 'hello'

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      "  optional string string_field = 1 [default='hello'];\n"
                      '}\n',
                      self.result)

  def testSingleFieldWithDefaultEmptyString(self):
    field = descriptor.FieldDescriptor()
    field.name = 'string_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.STRING
    field.default_value = ''

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      "  optional string string_field = 1 [default=''];\n"
                      '}\n',
                      self.result)

  def testSingleFieldWithDefaultMessage(self):
    field = descriptor.FieldDescriptor()
    field.name = 'message_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.MESSAGE
    field.type_name = 'MyNestedMessage'
    field.default_value = 'not valid'

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      "  optional MyNestedMessage message_field = 1;\n"
                      '}\n',
                      self.result)

  def testSingleFieldWithDefaultEnum(self):
    field = descriptor.FieldDescriptor()
    field.name = 'enum_field'
    field.number = 1
    field.label = descriptor.FieldDescriptor.Label.OPTIONAL
    field.variant = descriptor.FieldDescriptor.Variant.ENUM
    field.type_name = 'my_package.MyEnum'
    field.default_value = '17'

    self.MakeMessage(fields=[field])

    generate_proto.format_proto_file(self.file_descriptor, self.output)
    self.assertEquals('\n\n'
                      'message MyMessage {\n'
                      "  optional my_package.MyEnum enum_field = 1 "
                      "[default=17];\n"
                      '}\n',
                      self.result)


def main():
  unittest.main()


if __name__ == '__main__':
  main()




#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Common utility library."""

from __future__ import with_statement
import six

__author__ = ['rafek@google.com (Rafe Kaplan)',
              'guido@google.com (Guido van Rossum)',
]

import cgi
import datetime
import functools
import inspect
import os
import re
import sys

__all__ = ['AcceptItem',
           'AcceptError',
           'Error',
           'choose_content_type',
           'decode_datetime',
           'get_package_for_module',
           'pad_string',
           'parse_accept_header',
           'positional',
           'PROTORPC_PROJECT_URL',
           'TimeZoneOffset',
           'total_seconds',
]


class Error(Exception):
  """Base class for protorpc exceptions."""


class AcceptError(Error):
  """Raised when there is an error parsing the accept header."""


PROTORPC_PROJECT_URL = 'http://code.google.com/p/google-protorpc'

_TIME_ZONE_RE_STRING = r"""
  # Examples:
  #   +01:00
  #   -05:30
  #   Z12:00
  ((?P<z>Z) | (?P<sign>[-+])
   (?P<hours>\d\d) :
   (?P<minutes>\d\d))$
"""
_TIME_ZONE_RE = re.compile(_TIME_ZONE_RE_STRING, re.IGNORECASE | re.VERBOSE)


def pad_string(string):
  """Pad a string for safe HTTP error responses.

  Prevents Internet Explorer from displaying their own error messages
  when sent as the content of error responses.

  Args:
    string: A string.

  Returns:
    Formatted string left justified within a 512 byte field.
  """
  return string.ljust(512)


def positional(max_positional_args):
  """A decorator to declare that only the first N arguments may be positional.

  This decorator makes it easy to support Python 3 style keyword-only
  parameters. For example, in Python 3 it is possible to write:

    def fn(pos1, *, kwonly1=None, kwonly1=None):
      ...

  All named parameters after * must be a keyword:

    fn(10, 'kw1', 'kw2')  # Raises exception.
    fn(10, kwonly1='kw1')  # Ok.

  Example:
    To define a function like above, do:

      @positional(1)
      def fn(pos1, kwonly1=None, kwonly2=None):
        ...

    If no default value is provided to a keyword argument, it becomes a required
    keyword argument:

      @positional(0)
      def fn(required_kw):
        ...

    This must be called with the keyword parameter:

      fn()  # Raises exception.
      fn(10)  # Raises exception.
      fn(required_kw=10)  # Ok.

    When defining instance or class methods always remember to account for
    'self' and 'cls':

      class MyClass(object):

        @positional(2)
        def my_method(self, pos1, kwonly1=None):
          ...

        @classmethod
        @positional(2)
        def my_method(cls, pos1, kwonly1=None):
          ...

    One can omit the argument to 'positional' altogether, and then no
    arguments with default values may be passed positionally. This
    would be equivalent to placing a '*' before the first argument
    with a default value in Python 3. If there are no arguments with
    default values, and no argument is given to 'positional', an error
    is raised.

      @positional
      def fn(arg1, arg2, required_kw1=None, required_kw2=0):
        ...

      fn(1, 3, 5)  # Raises exception.
      fn(1, 3)  # Ok.
      fn(1, 3, required_kw1=5)  # Ok.

  Args:
    max_positional_arguments: Maximum number of positional arguments.  All
      parameters after the this index must be keyword only.

  Returns:
    A decorator that prevents using arguments after max_positional_args from
    being used as positional parameters.

  Raises:
    TypeError if a keyword-only argument is provided as a positional parameter.
    ValueError if no maximum number of arguments is provided and the function
      has no arguments with default values.
  """
  def positional_decorator(wrapped):
    @functools.wraps(wrapped)
    def positional_wrapper(*args, **kwargs):
      if len(args) > max_positional_args:
        plural_s = ''
        if max_positional_args != 1:
          plural_s = 's'
        raise TypeError('%s() takes at most %d positional argument%s '
                        '(%d given)' % (wrapped.__name__,
                                        max_positional_args,
                                        plural_s, len(args)))
      return wrapped(*args, **kwargs)
    return positional_wrapper

  if isinstance(max_positional_args, six.integer_types):
    return positional_decorator
  else:
    args, _, _, defaults = inspect.getargspec(max_positional_args)
    if defaults is None:
      raise ValueError(
          'Functions with no keyword arguments must specify '
          'max_positional_args')
    return positional(len(args) - len(defaults))(max_positional_args)


# TODO(rafek): Support 'level' from the Accept header standard.
class AcceptItem(object):
  """Encapsulate a single entry of an Accept header.

  Parses and extracts relevent values from an Accept header and implements
  a sort order based on the priority of each requested type as defined
  here:

    http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html

  Accept headers are normally a list of comma separated items.  Each item
  has the format of a normal HTTP header.  For example:

    Accept: text/plain, text/html, text/*, */*

  This header means to prefer plain text over HTML, HTML over any other
  kind of text and text over any other kind of supported format.

  This class does not attempt to parse the list of items from the Accept header.
  The constructor expects the unparsed sub header and the index within the
  Accept header that the fragment was found.

  Properties:
    index: The index that this accept item was found in the Accept header.
    main_type: The main type of the content type.
    sub_type: The sub type of the content type.
    q: The q value extracted from the header as a float.  If there is no q
      value, defaults to 1.0.
    values: All header attributes parsed form the sub-header.
    sort_key: A tuple (no_main_type, no_sub_type, q, no_values, index):
        no_main_type: */* has the least priority.
        no_sub_type: Items with no sub-type have less priority.
        q: Items with lower q value have less priority.
        no_values: Items with no values have less priority.
        index: Index of item in accept header is the last priority.
  """

  __CONTENT_TYPE_REGEX = re.compile(r'^([^/]+)/([^/]+)$')

  def __init__(self, accept_header, index):
    """Parse component of an Accept header.

    Args:
      accept_header: Unparsed sub-expression of accept header.
      index: The index that this accept item was found in the Accept header.
    """
    accept_header = accept_header.lower()
    content_type, values = cgi.parse_header(accept_header)
    match = self.__CONTENT_TYPE_REGEX.match(content_type)
    if not match:
      raise AcceptError('Not valid Accept header: %s' % accept_header)
    self.__index = index
    self.__main_type = match.group(1)
    self.__sub_type = match.group(2)
    self.__q = float(values.get('q', 1))
    self.__values = values

    if self.__main_type == '*':
      self.__main_type = None

    if self.__sub_type == '*':
      self.__sub_type = None

    self.__sort_key = (not self.__main_type,
                       not self.__sub_type,
                       -self.__q,
                       not self.__values,
                       self.__index)

  @property
  def index(self):
    return self.__index

  @property
  def main_type(self):
    return self.__main_type

  @property
  def sub_type(self):
    return self.__sub_type

  @property
  def q(self):
    return self.__q

  @property
  def values(self):
    """Copy the dictionary of values parsed from the header fragment."""
    return dict(self.__values)

  @property
  def sort_key(self):
    return self.__sort_key

  def match(self, content_type):
    """Determine if the given accept header matches content type.

    Args:
      content_type: Unparsed content type string.

    Returns:
      True if accept header matches content type, else False.
    """
    content_type, _ = cgi.parse_header(content_type)
    match = self.__CONTENT_TYPE_REGEX.match(content_type.lower())
    if not match:
      return False

    main_type, sub_type = match.group(1), match.group(2)
    if not(main_type and sub_type):
      return False

    return ((self.__main_type is None or self.__main_type == main_type) and
            (self.__sub_type is None or self.__sub_type == sub_type))


  def __cmp__(self, other):
    """Comparison operator based on sort keys."""
    if not isinstance(other, AcceptItem):
      return NotImplemented
    return cmp(self.sort_key, other.sort_key)

  def __str__(self):
    """Rebuilds Accept header."""
    content_type = '%s/%s' % (self.__main_type or '*', self.__sub_type or '*')
    values = self.values

    if values:
      value_strings = ['%s=%s' % (i, v) for i, v in values.items()]
      return '%s; %s' % (content_type, '; '.join(value_strings))
    else:
      return content_type

  def __repr__(self):
    return 'AcceptItem(%r, %d)' % (str(self), self.__index)


def parse_accept_header(accept_header):
  """Parse accept header.

  Args:
    accept_header: Unparsed accept header.  Does not include name of header.

  Returns:
    List of AcceptItem instances sorted according to their priority.
  """
  accept_items = []
  for index, header in enumerate(accept_header.split(',')):
    accept_items.append(AcceptItem(header, index))
  return sorted(accept_items)


def choose_content_type(accept_header, supported_types):
  """Choose most appropriate supported type based on what client accepts.

  Args:
    accept_header: Unparsed accept header.  Does not include name of header.
    supported_types: List of content-types supported by the server.  The index
      of the supported types determines which supported type is prefered by
      the server should the accept header match more than one at the same
      priority.

  Returns:
    The preferred supported type if the accept header matches any, else None.
  """
  for accept_item in parse_accept_header(accept_header):
    for supported_type in supported_types:
      if accept_item.match(supported_type):
        return supported_type
  return None


@positional(1)
def get_package_for_module(module):
  """Get package name for a module.

  Helper calculates the package name of a module.

  Args:
    module: Module to get name for.  If module is a string, try to find
      module in sys.modules.

  Returns:
    If module contains 'package' attribute, uses that as package name.
    Else, if module is not the '__main__' module, the module __name__.
    Else, the base name of the module file name.  Else None.
  """
  if isinstance(module, six.string_types):
    try:
      module = sys.modules[module]
    except KeyError:
      return None

  try:
    return six.text_type(module.package)
  except AttributeError:
    if module.__name__ == '__main__':
      try:
        file_name = module.__file__
      except AttributeError:
        pass
      else:
        base_name = os.path.basename(file_name)
        split_name = os.path.splitext(base_name)
        if len(split_name) == 1:
          return six.text_type(base_name)
        else:
          return u'.'.join(split_name[:-1])

    return six.text_type(module.__name__)


def total_seconds(offset):
  """Backport of offset.total_seconds() from python 2.7+."""
  seconds = offset.days * 24 * 60 * 60 + offset.seconds
  microseconds = seconds * 10**6 + offset.microseconds
  return microseconds / (10**6 * 1.0)


class TimeZoneOffset(datetime.tzinfo):
  """Time zone information as encoded/decoded for DateTimeFields."""

  def __init__(self, offset):
    """Initialize a time zone offset.

    Args:
      offset: Integer or timedelta time zone offset, in minutes from UTC.  This
        can be negative.
    """
    super(TimeZoneOffset, self).__init__()
    if isinstance(offset, datetime.timedelta):
      offset = total_seconds(offset) / 60
    self.__offset = offset

  def utcoffset(self, dt):
    """Get the a timedelta with the time zone's offset from UTC.

    Returns:
      The time zone offset from UTC, as a timedelta.
    """
    return datetime.timedelta(minutes=self.__offset)

  def dst(self, dt):
    """Get the daylight savings time offset.

    The formats that ProtoRPC uses to encode/decode time zone information don't
    contain any information about daylight savings time.  So this always
    returns a timedelta of 0.

    Returns:
      A timedelta of 0.
    """
    return datetime.timedelta(0)


def decode_datetime(encoded_datetime):
  """Decode a DateTimeField parameter from a string to a python datetime.

  Args:
    encoded_datetime: A string in RFC 3339 format.

  Returns:
    A datetime object with the date and time specified in encoded_datetime.

  Raises:
    ValueError: If the string is not in a recognized format.
  """
  # Check if the string includes a time zone offset.  Break out the
  # part that doesn't include time zone info.  Convert to uppercase
  # because all our comparisons should be case-insensitive.
  time_zone_match = _TIME_ZONE_RE.search(encoded_datetime)
  if time_zone_match:
    time_string = encoded_datetime[:time_zone_match.start(1)].upper()
  else:
    time_string = encoded_datetime.upper()

  if '.' in time_string:
    format_string = '%Y-%m-%dT%H:%M:%S.%f'
  else:
    format_string = '%Y-%m-%dT%H:%M:%S'

  decoded_datetime = datetime.datetime.strptime(time_string, format_string)

  if not time_zone_match:
    return decoded_datetime

  # Time zone info was included in the parameter.  Add a tzinfo
  # object to the datetime.  Datetimes can't be changed after they're
  # created, so we'll need to create a new one.
  if time_zone_match.group('z'):
    offset_minutes = 0
  else:
    sign = time_zone_match.group('sign')
    hours, minutes = [int(value) for value in
                      time_zone_match.group('hours', 'minutes')]
    offset_minutes = hours * 60 + minutes
    if sign == '-':
      offset_minutes *= -1

  return datetime.datetime(decoded_datetime.year,
                           decoded_datetime.month,
                           decoded_datetime.day,
                           decoded_datetime.hour,
                           decoded_datetime.minute,
                           decoded_datetime.second,
                           decoded_datetime.microsecond,
                           TimeZoneOffset(offset_minutes))



#!/usr/bin/env python
#
# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""
This is the Python counterpart to the RawMessage class defined in rawmessage.h.

To use this, put the following line in your .proto file:

python from google.net.proto.RawMessage import RawMessage

"""

__pychecker__ = 'no-callinit no-argsused'

from google.net.proto import ProtocolBuffer

class RawMessage(ProtocolBuffer.ProtocolMessage):
  """
  This is a special subclass of ProtocolMessage that doesn't interpret its data
  in any way. Instead, it just stores it in a string.

  See rawmessage.h for more details.
  """

  def __init__(self, initial=None):
    self.__contents = ''
    if initial is not None:
      self.MergeFromString(initial)

  def contents(self):
    return self.__contents

  def set_contents(self, contents):
    self.__contents = contents

  def Clear(self):
    self.__contents = ''

  def IsInitialized(self, debug_strs=None):
    return 1

  def __str__(self, prefix="", printElemNumber=0):
    return prefix + self.DebugFormatString(self.__contents)

  def OutputUnchecked(self, e):
    e.putRawString(self.__contents)

  def OutputPartial(self, e):
    return self.OutputUnchecked(e)

  def TryMerge(self, d):
    self.__contents = d.getRawString()

  def MergeFrom(self, pb):
    assert pb is not self
    if pb.__class__ != self.__class__:
      return 0
    self.__contents = pb.__contents
    return 1

  def Equals(self, pb):
    return self.__contents == pb.__contents

  def __eq__(self, other):
    return (other is not None) and (other.__class__ == self.__class__) and self.Equals(other)

  def __ne__(self, other):
    return not (self == other)

  def ByteSize(self):
    return len(self.__contents)

  def ByteSizePartial(self):
    return self.ByteSize()



#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""WSGI application tests."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import unittest


from protorpc import end2end_test
from protorpc import protojson
from protorpc import remote
from protorpc import registry
from protorpc import transport
from protorpc import test_util
from protorpc import webapp_test_util
from protorpc.wsgi import service
from protorpc.wsgi import util


class ServiceMappingTest(end2end_test.EndToEndTest):

  def setUp(self):
    self.protocols = None
    remote.Protocols.set_default(remote.Protocols.new_default())
    super(ServiceMappingTest, self).setUp()

  def CreateServices(self):

    return my_service, my_other_service

  def  CreateWsgiApplication(self):
    """Create WSGI application used on the server side for testing."""
    my_service = service.service_mapping(webapp_test_util.TestService,
                                         '/my/service')
    my_other_service = service.service_mapping(
      webapp_test_util.TestService.new_factory('initialized'),
      '/my/other_service',
      protocols=self.protocols)

    return util.first_found([my_service, my_other_service])

  def testAlternateProtocols(self):
    self.protocols = remote.Protocols()
    self.protocols.add_protocol(protojson, 'altproto', 'image/png')

    global_protocols = remote.Protocols()
    global_protocols.add_protocol(protojson, 'server-side-name', 'image/png')
    remote.Protocols.set_default(global_protocols)
    self.ResetServer()

    self.connection = transport.HttpTransport(
      self.service_url, protocol=self.protocols.lookup_by_name('altproto'))
    self.stub = webapp_test_util.TestService.Stub(self.connection)

    self.stub.optional_message(string_value='alternate-protocol')

  def testAlwaysUseDefaults(self):
    new_protocols = remote.Protocols()
    new_protocols.add_protocol(protojson, 'altproto', 'image/png')

    self.connection = transport.HttpTransport(
      self.service_url, protocol=new_protocols.lookup_by_name('altproto'))
    self.stub = webapp_test_util.TestService.Stub(self.connection)

    self.assertRaisesWithRegexpMatch(
      remote.ServerError,
      'HTTP Error 415: Unsupported Media Type',
      self.stub.optional_message, string_value='alternate-protocol')

    remote.Protocols.set_default(new_protocols)

    self.stub.optional_message(string_value='alternate-protocol')


class ProtoServiceMappingsTest(ServiceMappingTest):

  def  CreateWsgiApplication(self):
    """Create WSGI application used on the server side for testing."""
    return service.service_mappings(
      [('/my/service', webapp_test_util.TestService),
       ('/my/other_service',
        webapp_test_util.TestService.new_factory('initialized'))
      ])

  def GetRegistryStub(self, path='/protorpc'):
    service_url = self.make_service_url(path)
    transport = self.CreateTransport(service_url)
    return registry.RegistryService.Stub(transport)

  def testRegistry(self):
    registry_client = self.GetRegistryStub()
    response = registry_client.services()
    self.assertIterEqual([
        registry.ServiceMapping(
            name='/my/other_service',
            definition='protorpc.webapp_test_util.TestService'),
        registry.ServiceMapping(
            name='/my/service',
            definition='protorpc.webapp_test_util.TestService'),
        ], response.services)

  def testRegistryDictionary(self):
    self.ResetServer(service.service_mappings(
      {'/my/service': webapp_test_util.TestService,
       '/my/other_service':
           webapp_test_util.TestService.new_factory('initialized'),
      }))
    registry_client = self.GetRegistryStub()
    response = registry_client.services()
    self.assertIterEqual([
        registry.ServiceMapping(
            name='/my/other_service',
            definition='protorpc.webapp_test_util.TestService'),
        registry.ServiceMapping(
            name='/my/service',
            definition='protorpc.webapp_test_util.TestService'),
        ], response.services)

  def testNoRegistry(self):
    self.ResetServer(service.service_mappings(
      [('/my/service', webapp_test_util.TestService),
       ('/my/other_service',
        webapp_test_util.TestService.new_factory('initialized'))
      ],
      registry_path=None))
    registry_client = self.GetRegistryStub()
    self.assertRaisesWithRegexpMatch(
      remote.ServerError,
      'HTTP Error 404: Not Found',
      registry_client.services)

  def testAltRegistry(self):
    self.ResetServer(service.service_mappings(
      [('/my/service', webapp_test_util.TestService),
       ('/my/other_service',
        webapp_test_util.TestService.new_factory('initialized'))
      ],
      registry_path='/registry'))
    registry_client = self.GetRegistryStub('/registry')
    services = registry_client.services()
    self.assertTrue(isinstance(services, registry.ServicesResponse))
    self.assertIterEqual(
        [registry.ServiceMapping(
            name='/my/other_service',
            definition='protorpc.webapp_test_util.TestService'),
         registry.ServiceMapping(
             name='/my/service',
             definition='protorpc.webapp_test_util.TestService'),
        ],
        services.services)

  def testDuplicateRegistryEntry(self):
    self.assertRaisesWithRegexpMatch(
      remote.ServiceConfigurationError,
      "Path '/my/service' is already defined in service mapping",
      service.service_mappings,
      [('/my/service', webapp_test_util.TestService),
       ('/my/service',
        webapp_test_util.TestService.new_factory('initialized'))
      ])

  def testRegex(self):
    self.ResetServer(service.service_mappings(
      [('/my/[0-9]+', webapp_test_util.TestService.new_factory('service')),
       ('/my/[a-z]+',
            webapp_test_util.TestService.new_factory('other-service')),
      ]))
    my_service_url = 'http://localhost:%d/my/12345' % self.port
    my_other_service_url = 'http://localhost:%d/my/blarblar' % self.port

    my_service = webapp_test_util.TestService.Stub(
      transport.HttpTransport(my_service_url))
    my_other_service = webapp_test_util.TestService.Stub(
      transport.HttpTransport(my_other_service_url))

    response = my_service.init_parameter()
    self.assertEqual('service', response.string_value)

    response = my_other_service.init_parameter()
    self.assertEqual('other-service', response.string_value)


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#





"""This module contains the MessageSet class, which is a special kind of
protocol message which can contain other protocol messages without knowing
their types.  See the class's doc string for more information."""








from google.net.proto import ProtocolBuffer
import logging



try:
  from google3.net.proto import _net_proto___parse__python
except ImportError:
  _net_proto___parse__python = None


TAG_BEGIN_ITEM_GROUP = 11
TAG_END_ITEM_GROUP   = 12
TAG_TYPE_ID          = 16
TAG_MESSAGE          = 26

class Item:

  def __init__(self, message, message_class=None):
    self.message = message
    self.message_class = message_class

  def SetToDefaultInstance(self, message_class):
    self.message = message_class()
    self.message_class = message_class

  def Parse(self, message_class):

    if self.message_class is not None:
      return 1

    try:
      message_obj = message_class()
      message_obj.MergePartialFromString(self.message)
      self.message = message_obj
      self.message_class = message_class
      return 1
    except ProtocolBuffer.ProtocolBufferDecodeError:
      logging.warn("Parse error in message inside MessageSet.  Tried "
                   "to parse as: " + message_class.__name__)
      return 0

  def MergeFrom(self, other):

    if self.message_class is not None:
      if other.Parse(self.message_class):
        self.message.MergeFrom(other.message)



    elif other.message_class is not None:
      if not self.Parse(other.message_class):

        self.message = other.message_class()
        self.message_class = other.message_class
      self.message.MergeFrom(other.message)

    else:
      self.message += other.message

  def Copy(self):

    if self.message_class is None:
      return Item(self.message)
    else:
      new_message = self.message_class()
      new_message.CopyFrom(self.message)
      return Item(new_message, self.message_class)

  def Equals(self, other):

    if self.message_class is not None:
      if not other.Parse(self.message_class): return 0
      return self.message.Equals(other.message)

    elif other.message_class is not None:
      if not self.Parse(other.message_class): return 0
      return self.message.Equals(other.message)

    else:
      return self.message == other.message

  def IsInitialized(self, debug_strs=None):

    if self.message_class is None:
      return 1
    else:
      return self.message.IsInitialized(debug_strs)

  def ByteSize(self, pb, type_id):

    message_length = 0
    if self.message_class is None:
      message_length = len(self.message)
    else:
      message_length = self.message.ByteSize()


    return pb.lengthString(message_length) + pb.lengthVarInt64(type_id) + 2

  def ByteSizePartial(self, pb, type_id):

    message_length = 0
    if self.message_class is None:
      message_length = len(self.message)
    else:
      message_length = self.message.ByteSizePartial()


    return pb.lengthString(message_length) + pb.lengthVarInt64(type_id) + 2

  def OutputUnchecked(self, out, type_id):

    out.putVarInt32(TAG_TYPE_ID)




    out.putVarUint64(type_id)
    out.putVarInt32(TAG_MESSAGE)
    if self.message_class is None:
      out.putPrefixedString(self.message)
    else:
      out.putVarInt32(self.message.ByteSize())
      self.message.OutputUnchecked(out)

  def OutputPartial(self, out, type_id):

    out.putVarInt32(TAG_TYPE_ID)




    out.putVarUint64(type_id)
    out.putVarInt32(TAG_MESSAGE)
    if self.message_class is None:
      out.putPrefixedString(self.message)
    else:
      out.putVarInt32(self.message.ByteSizePartial())
      self.message.OutputPartial(out)

  def Decode(decoder):

    type_id = 0
    message = None
    while 1:
      tag = decoder.getVarInt32()
      if tag == TAG_END_ITEM_GROUP:
        break
      if tag == TAG_TYPE_ID:




        type_id = decoder.getVarUint64()
        continue
      if tag == TAG_MESSAGE:
        message = decoder.getPrefixedString()
        continue


      if tag == 0: raise ProtocolBuffer.ProtocolBufferDecodeError
      decoder.skipData(tag)

    if type_id == 0 or message is None:
      raise ProtocolBuffer.ProtocolBufferDecodeError
    return (type_id, message)
  Decode = staticmethod(Decode)


class MessageSet(ProtocolBuffer.ProtocolMessage):

  def __init__(self, contents=None):
    self.items = dict()
    if contents is not None: self.MergeFromString(contents)




  def get(self, message_class):

    if message_class.MESSAGE_TYPE_ID not in self.items:
      return message_class()
    item = self.items[message_class.MESSAGE_TYPE_ID]
    if item.Parse(message_class):
      return item.message
    else:
      return message_class()

  def mutable(self, message_class):

    if message_class.MESSAGE_TYPE_ID not in self.items:
      message = message_class()
      self.items[message_class.MESSAGE_TYPE_ID] = Item(message, message_class)
      return message
    item = self.items[message_class.MESSAGE_TYPE_ID]
    if not item.Parse(message_class):
      item.SetToDefaultInstance(message_class)
    return item.message

  def has(self, message_class):

    if message_class.MESSAGE_TYPE_ID not in self.items:
      return 0
    item = self.items[message_class.MESSAGE_TYPE_ID]
    return item.Parse(message_class)

  def has_unparsed(self, message_class):
    return message_class.MESSAGE_TYPE_ID in self.items

  def GetTypeIds(self):
    return list(self.items.keys())

  def NumMessages(self):
    return len(self.items)

  def remove(self, message_class):
    if message_class.MESSAGE_TYPE_ID in self.items:
      del self.items[message_class.MESSAGE_TYPE_ID]




  def __getitem__(self, message_class):
    if message_class.MESSAGE_TYPE_ID not in self.items:
      raise KeyError(message_class)
    item = self.items[message_class.MESSAGE_TYPE_ID]
    if item.Parse(message_class):
      return item.message
    else:
      raise KeyError(message_class)

  def __setitem__(self, message_class, message):
    self.items[message_class.MESSAGE_TYPE_ID] = Item(message, message_class)

  def __contains__(self, message_class):
    return self.has(message_class)

  def __delitem__(self, message_class):
    self.remove(message_class)

  def __len__(self):
    return len(self.items)




  def MergeFrom(self, other):

    assert other is not self

    for (type_id, item) in list(other.items.items()):
      if type_id in self.items:
        self.items[type_id].MergeFrom(item)
      else:
        self.items[type_id] = item.Copy()

  def Equals(self, other):
    if other is self: return 1
    if len(self.items) != len(other.items): return 0

    for (type_id, item) in list(other.items.items()):
      if type_id not in self.items: return 0
      if not self.items[type_id].Equals(item): return 0

    return 1

  def __eq__(self, other):
    return ((other is not None)
        and (other.__class__ == self.__class__)
        and self.Equals(other))

  def __ne__(self, other):
    return not (self == other)

  def IsInitialized(self, debug_strs=None):

    initialized = 1
    for item in list(self.items.values()):
      if not item.IsInitialized(debug_strs):
        initialized = 0
    return initialized

  def ByteSize(self):
    n = 2 * len(self.items)
    for (type_id, item) in list(self.items.items()):
      n += item.ByteSize(self, type_id)
    return n

  def ByteSizePartial(self):
    n = 2 * len(self.items)
    for (type_id, item) in list(self.items.items()):
      n += item.ByteSizePartial(self, type_id)
    return n

  def Clear(self):
    self.items = dict()

  def OutputUnchecked(self, out):
    for (type_id, item) in list(self.items.items()):
      out.putVarInt32(TAG_BEGIN_ITEM_GROUP)
      item.OutputUnchecked(out, type_id)
      out.putVarInt32(TAG_END_ITEM_GROUP)

  def OutputPartial(self, out):
    for (type_id, item) in list(self.items.items()):
      out.putVarInt32(TAG_BEGIN_ITEM_GROUP)
      item.OutputPartial(out, type_id)
      out.putVarInt32(TAG_END_ITEM_GROUP)

  def TryMerge(self, decoder):
    while decoder.avail() > 0:
      tag = decoder.getVarInt32()
      if tag == TAG_BEGIN_ITEM_GROUP:
        (type_id, message) = Item.Decode(decoder)
        if type_id in self.items:
          self.items[type_id].MergeFrom(Item(message))
        else:
          self.items[type_id] = Item(message)
        continue


      if (tag == 0): raise ProtocolBuffer.ProtocolBufferDecodeError
      decoder.skipData(tag)


  def _CToASCII(self, output_format):
    if _net_proto___parse__python is None:
      return ProtocolBuffer.ProtocolMessage._CToASCII(self, output_format)
    else:
      return _net_proto___parse__python.ToASCII(
          self, "MessageSetInternal", output_format)

  def ParseASCII(self, s):
    if _net_proto___parse__python is None:
      ProtocolBuffer.ProtocolMessage.ParseASCII(self, s)
    else:
      _net_proto___parse__python.ParseASCII(self, "MessageSetInternal", s)

  def ParseASCIIIgnoreUnknown(self, s):
    if _net_proto___parse__python is None:
      ProtocolBuffer.ProtocolMessage.ParseASCIIIgnoreUnknown(self, s)
    else:
      _net_proto___parse__python.ParseASCIIIgnoreUnknown(
          self, "MessageSetInternal", s)







  def __str__(self, prefix="", printElemNumber=0):
    text = ""
    for (type_id, item) in list(self.items.items()):
      if item.message_class is None:
        text += "%s[%d] <\n" % (prefix, type_id)
        text += "%s  (%d bytes)\n" % (prefix, len(item.message))
        text += "%s>\n" % prefix
      else:
        text += "%s[%s] <\n" % (prefix, item.message_class.__name__)
        text += item.message.__str__(prefix + "  ", printElemNumber)
        text += "%s>\n" % prefix
    return text


  _PROTO_DESCRIPTOR_NAME = 'MessageSet'

__all__ = ['MessageSet']
